(*
 *                                                                                             .
 * Module name: Fakultaet.pas; Module type: Delphi unit; Language: Object pascal;              .
 * Original developed with: Delphi 5 Enterprise; Portability: most likely down to Delphi 2;    .
 *                                                                                             .
 * Author: ~SoffyT (web:<http://www.nefzger.at/~SoffyT/>; mailto:<MissyQueen@nefzger.at>)      .
 * File created: Fri., 01.11.2002; Last modified: Fri., 17.01.2003                             .
 *                                                                                             .
 * Bug report: Pls. report all bugs, wishes, suggestions etc. to <bugreport@nefzger.at>.       .
 *                                                                                             .
 *  Purpose:                                                                                   .
 * ----------                                                                                  .
 *                                                                                             .
 *  Implemented classes:                                                                       .
 * ----------------------                                                                      .
 *    TFakultaet { = class (TObject) }                                                         .
 *                                                                                             .
 *)
unit
   Fakultaet;

// ----------                                                                                  .
   interface
// ----------                                                                                  .

type
   PArrayOfByte = ^TArrayOfByte;
   TArrayOfByte = packed array of Byte;

type
   TFakultaet = class
   { Private-Deklarationen }
   private
      FMethode: Integer;
      FN: Integer;
      FNErg: TArrayOfByte;
      FNErgLen: Integer;
      FMultiplyIterations: Int64;
      FUpdateCarryIterations: Int64;
      procedure Calculate_1 ({keine Parameter});
      procedure Calculate_2 ({keine Parameter});
      procedure Calculate_3 ({keine Parameter});
      procedure SetN (const AN: Integer);
      function GetResult ({keine Parameter}): String;
   { Protected-Deklarationen }
   protected
   { Public-Deklarationen }
   public
      constructor Create ({keine Parameter});
      destructor Destroy ({keine Parameter}); override;
      property M: Integer read FMethode write FMethode;
      property N: Integer read FN write SetN;
      property NumDigits: Integer read FNErgLen;
      property M_Iterations: Int64 read FMultiplyIterations;
      property UC_Iterations: Int64 read FUpdateCarryIterations;
      property Result: String read GetResult;
      class function GetBestM: Integer;
   { Published-Deklarationen }
   published
   end;

// ---------------                                                                             .
   implementation
// ---------------                                                                             .

const
   ERR_NO_SUCH_METHODE   = -1;
   ERR_BUFFER_OVERFLOW_1 = -2;
   ERR_BUFFER_OVERFLOW_2 = -3;

const
   MAX = (8*(1024*1024)); // Max. Anzahl an Ziffern                                            .
   NAN = Byte(-1);        // 'Flag' fr ~N~ot ~A~ ~N~umber                                     .

// ----------------------------------------------------------------------------                .
// Berechnung von N! - Erste (AKA "originale") Implementierung                                 .
// ----------------------------------------------------------------------------                .
procedure TFakultaet.Calculate_1 ();
var
   i, c: Integer;
begin
   // Grundinitialisierung                                                                     .
   for i := High(FNErg) downto 0 do
      FNErg[i] := NAN;
   // Ergebnis fr 1! initialisieren                                                           .
   FNErg[0] := 1;
   // Ergebnis fr N! berechnen                                                                .
   if (FN > 1) then begin
      i := 2;
      while (i <= FN) do begin
         FNErgLen := 0;                // Index                                                .
         c := 0;                       // (Teil-) Ergebnis; akkumulativ; eine Art "Carry"      .
         // Berechne: (Bisheriges Ergebnis) * i                                                .
         while ((FNErgLen <= High(FNErg)) and (FNErg[FNErgLen] <> NAN)) do begin
            c := ((i * Integer(FNErg[FNErgLen])) + c);
            FNErg[FNErgLen] := Byte(c mod 10);
            c := (c div 10);
            Inc(FNErgLen);
            Inc(FMultiplyIterations);
         end;
         // Bei Buffer-berlauf: Fehler setzen, raus hier                                      .
         if (FNErgLen > High(FNErg)) then begin
            FNErgLen := ERR_BUFFER_OVERFLOW_1;
            Exit;
         end;
         // Aktualisiere: angelaufener bertrag c                                              .
         while ((FNErgLen <= High(FNErg)) and (c > 0)) do begin
            FNErg[FNErgLen] := Byte(c mod 10);
            c := (c div 10);
            Inc(FNErgLen);
            Inc(FUpdateCarryIterations);
         end;
         // Bei Buffer-berlauf: Fehler setzen, raus hier                                      .
         if (FNErgLen > High(FNErg)) then begin
            FNErgLen := ERR_BUFFER_OVERFLOW_2;
            Exit;
         end;
         Inc(i);
      end;
   end else
      FNErgLen := 1;
end;

// ----------------------------------------------------------------------------                .
// Berechnung von N! - Zweite Implementierung:                                                 .
//    High(FNErg) wurde durch (MAX-1) ersetzt, da das Feld *IMMER*                             .
//    MAX Byte gro ist (indiziert von [0 .. (MAX-1)]                                          .
// ----------------------------------------------------------------------------                .
procedure TFakultaet.Calculate_2 ();
var
   i, c: Integer;
begin
   // Grundinitialisierung                                                                     .
   for i := (MAX - 1) downto 0 do
      FNErg[i] := NAN;
   // Ergebnis fr 1! initialisieren                                                           .
   FNErg[0] := 1;
   // Ergebnis fr N! berechnen                                                                .
   if (FN > 1) then begin
      i := 2;
      while (i <= FN) do begin
         FNErgLen := 0;                // Index                                                .
         c := 0;                       // (Teil-) Ergebnis; akkumulativ; eine Art "Carry"      .
         // Berechne: (Bisheriges Ergebnis) * i                                                .
         while ((FNErgLen <= (MAX-1)) and (FNErg[FNErgLen] <> NAN)) do begin
            c := ((i*Integer(FNErg[FNErgLen])) + c);
            FNErg[FNErgLen] := Byte(c mod 10);
            c := (c div 10);
            Inc(FNErgLen);
            Inc(FMultiplyIterations);
         end;
         // Bei Buffer-berlauf: Fehler setzen, raus hier                                      .
         if (FNErgLen > (MAX-1)) then begin
            FNErgLen := ERR_BUFFER_OVERFLOW_1;
            Exit;
         end;
         // Aktualisiere: angelaufener bertrag c                                              .
         while ((FNErgLen <= (MAX-1)) and (c > 0)) do begin
            FNErg[FNErgLen] := Byte(c mod 10);
            c := (c div 10);
            Inc(FNErgLen);
            Inc(FUpdateCarryIterations);
         end;
         // Bei Buffer-berlauf: Fehler setzen, raus hier                                      .
         if (FNErgLen > (MAX-1)) then begin
            FNErgLen := ERR_BUFFER_OVERFLOW_2;
            Exit;
         end;
         Inc(i);
      end;
   end else
      FNErgLen := 1;
end;

// ----------------------------------------------------------------------------                .
// Berechnung von N! - Zweite Implementierung:                                                 .
//    High(FNErg) wurde durch (MAX-1) ersetzt, da das Feld *IMMER*                             .
//    MAX Byte gro ist (indiziert von [0 .. (MAX-1)]                                          .
// ----------------------------------------------------------------------------                .
procedure TFakultaet.Calculate_3 ();
var
   i, c: Integer;
begin
   // Grundinitialisierung                                                                     .
   for i := (MAX - 1) downto 0 do
      FNErg[i] := NAN;
   // Ergebnis fr 1! initialisieren                                                           .
   FNErg[0] := 1;
   // Ergebnis fr N! berechnen                                                                .
   if (FN > 1) then begin
      i := 2;
      while (i <= FN) do begin
         FNErgLen := 0;                // Index                                                .
         c := 0;                       // (Teil-) Ergebnis; akkumulativ; eine Art "Carry"      .
         // Berechne: (Bisheriges Ergebnis) * i                                                .
         while (FNErg[FNErgLen] <> NAN) do begin
            c := ((i * Integer(FNErg[FNErgLen])) + c);
            FNErg[FNErgLen] := Byte(c mod 10);
            c := (c div 10);
            Inc(FNErgLen);
            Inc(FMultiplyIterations);
         end;
         // Aktualisiere: angelaufener bertrag c                                              .
         while (c > 0) do begin
            FNErg[FNErgLen] := Byte(c mod 10);
            c := (c div 10);
            Inc(FNErgLen);
            Inc(FUpdateCarryIterations);
         end;
         Inc(i);
      end;
   end else
      FNErgLen := 1;
end;

// ----------------------------------------------------------------------------                .
procedure TFakultaet.SetN (const AN: Integer);
begin
   if ((AN > 0) and (AN <> FN)) then begin
      FN := AN;
      FMultiplyIterations := 0;
      FUpdateCarryIterations := 0;
      case FMethode of                 // N! (sprich: 'N-Fakultt') berechnen                  .
         1: Calculate_1();
         2: Calculate_2();
         3: Calculate_3();
         else
            FNErgLen := ERR_NO_SUCH_METHODE;
      end;
   end;
end;

// ----------------------------------------------------------------------------                .
function TFakultaet.GetResult (): String;
var
   i, j: Integer;
begin
   if (FNErgLen < 0) then begin
      case FNErgLen of
         ERR_NO_SUCH_METHODE:
            Result := 'No such methode';
         ERR_BUFFER_OVERFLOW_1:
            Result := 'Digit buffer overflow while in loop #1 (calculating result)';
         ERR_BUFFER_OVERFLOW_2:
            Result := 'Digit buffer overflow while in loop #2 (updating result /w carry)';
         else
            Result := '';
      end;
   end else
      if (FN <= 0) then
         Result := '1'                 // negative Werte und '0' sind '1'                      .
      else begin
         i := 0;
         j := FNErgLen;
         SetLength(Result,j);
         while (i < FNErgLen) do begin // sonst: Ergebnis zu String zusammensetzen             .
            Result[j] := Chr(Ord('0') + FNErg[i]);
            Inc(i);
            Dec(j);
         end;
      end;
end;

// ----------------------------------------------------------------------------                .
constructor TFakultaet.Create ();
begin
   inherited;                          // Initialisierung:                                     .
   FMethode := TFakultaet.GetBestM();  //    .) beste (= schnellste) Implementierung           .
   FN := (-1);                         //    .) keine Fakultt                                 .
   SetLength(FNErg,MAX);               //    .) Speicherbuffer (fr Berechnung) allokieren     .
   FNErgLen := 0;                      //    .) keine Ergebnislnge                            .
   FMultiplyIterations := 0;
   FUpdateCarryIterations := 0;
end;

// ----------------------------------------------------------------------------                .
destructor TFakultaet.Destroy ();
begin
   SetLength(FNErg,0);                 // Shutdown:                                            .
   inherited;                          //    .) Speicherbuffer freigeben                       .
end;

// ----------------------------------------------------------------------------                .
class function TFakultaet.GetBestM: Integer;
begin
   Result := 3; // Zugleich auch die "hchste" Methode:                                          .
              //    gltig sind 1 .. GetBestM                                                  .
end;

   // keine Initialisierung.                                                                   .
end.

