(*
 *                                                                                             .
 * Module name: Misc_Utilities.pas; Module type: Delphi unit; Language: Object pascal;         .
 * Original developed with: Delphi 5 Enterprise; Portability: most likely down to Delphi 2;    .
 *                                                                                             .
 * Author: Marcus F. Nefzger AKA MNe (web:<http://www.nefzger.at/>; mailto:<MNe@nefzger.at>)   .
 * File created: Fri., 01.11.2002; Last modified: 02.08.2005 @ 10:42                           .
 *                                                                                             .
 * Bug report: Pls. report all bugs, wishes, suggestions etc. to <BugReport@nefzger.at>.       .
 *                                                                                             .
 *  Purpose:                                                                                   .
 * ----------                                                                                  .
 *                                                                                             .
 *)

unit
   Misc_Utilities;

{$INCLUDE '..\includes\Globals.inc'}

// ----------                                                                                  .
   interface
// ----------                                                                                  .

uses
   Windows;

// -----------------------------------------------------------------------------               .

const

(* ==========================================================================

   http://www.neurophys.wisc.edu/www/comp/docs/ascii.html

   A S C I I   T a b l e   ( 7 - b i  t )
   --------------------------------------
   (ASCII = American Standard Code for Information Interchange)

   +---+---+---+--------+---+-------------------------++---+---+---+--------+---+-------------------------+
   |Dec|Oct|Hex|Binary  |Val|Description              ||Dec|Oct|Hex|Binary  |Val|Description              |
   +---+---+---+--------+---+-------------------------++---+---+---+--------+---+-------------------------+
   |000|000|000|00000000|NUL|Null character           ||064|100|040|01000000|@  |AT symbol                |
   |001|001|001|00000001|SOH|Start of Header          ||065|101|041|01000001|A  |                         |
   |002|002|002|00000010|STX|Start of Text            ||066|102|042|01000010|B  |                         |
   |003|003|003|00000011|ETX|End of Text              ||067|103|043|01000011|C  |                         |
   |004|004|004|00000100|EOT|End of Transmission      ||068|104|044|01000100|D  |                         |
   |005|005|005|00000101|ENQ|Enquiry                  ||069|105|045|01000101|E  |                         |
   |006|006|006|00000110|ACK|Acknowledgment           ||070|106|046|01000110|F  |                         |
   |007|007|007|00000111|BEL|Bell                     ||071|107|047|01000111|G  |                         |
   |008|010|008|00001000|BS |Backspace                ||072|110|048|01001000|H  |                         |
   |009|011|009|00001001|HT |Horizontal Tab           ||073|111|049|01001001|I  |                         |
   |010|012|00A|00001010|LF |Line Feed                ||074|112|04A|01001010|J  |                         |
   |011|013|00B|00001011|VT |Vertical Tab             ||075|113|04B|01001011|K  |                         |
   |012|014|00C|00001100|FF |Form Feed                ||076|114|04C|01001100|L  |                         |
   |013|015|00D|00001101|CR |Carriage Return          ||077|115|04D|01001101|M  |                         |
   |014|016|00E|00001110|SO |Shift Out                ||078|116|04E|01001110|N  |                         |
   |015|017|00F|00001111|SI |Shift In                 ||079|117|04F|01001111|O  |                         |
   |016|020|010|00010000|DLE|Data Link Escape         ||080|120|050|01010000|P  |                         |
   |017|021|011|00010001|DC1|XON  (Device Control 1   ||081|121|051|01010001|Q  |                         |
   |018|022|012|00010010|DC2|Device Control 2         ||082|122|052|01010010|R  |                         |
   |019|023|013|00010011|DC3|XOFF (Device Control 3   ||083|123|053|01010011|S  |                         |
   |020|024|014|00010100|DC4|Device Control 4         ||084|124|054|01010100|T  |                         |
   |021|025|015|00010101|NAK|Negative Acknowledgement ||085|125|055|01010101|U  |                         |
   |022|026|016|00010110|SYN|Synchronous Idle         ||086|126|056|01010110|V  |                         |
   |023|027|017|00010111|ETB|End of Transmission Block||087|127|057|01010111|W  |                         |
   |024|030|018|00011000|CAN|Cancel                   ||088|130|058|01011000|X  |                         |
   |025|031|019|00011001|EM |End of Medium            ||089|131|059|01011001|Y  |                         |
   |026|032|01A|00011010|SUB|Substitute               ||090|132|05A|01011010|Z  |                         |
   |027|033|01B|00011011|ESC|Escape                   ||091|133|05B|01011011|[  |left/opening bracket     |
   |028|034|01C|00011100|FS |File Separator           ||092|134|05C|01011100|\  |back slash               |
   |029|035|01D|00011101|GS |Group Separator          ||093|135|05D|01011101|]  |right/closing bracket    |
   |030|036|01E|00011110|RS |Request to Send          ||094|136|05E|01011110|^  |caret/cirumflex          |
   |031|037|01F|00011111|US |Unit Separator           ||095|137|05F|01011111|_  |underscore               |
   |032|040|020|00100000|SP |Space                    ||096|140|060|01100000|`  |                         |
   |033|041|021|00100001|!  |exclamation mark         ||097|141|061|01100001|a  |                         |
   |034|042|022|00100010|"  |double quote             ||098|142|062|01100010|b  |                         |
   |035|043|023|00100011|#  |number sign              ||099|143|063|01100011|c  |                         |
   |036|044|024|00100100|$  |dollar sign              ||100|144|064|01100100|d  |                         |
   |037|045|025|00100101|%  |percent                  ||101|145|065|01100101|e  |                         |
   |038|046|026|00100110|&  |ampersand                ||102|146|066|01100110|f  |                         |
   |039|047|027|00100111|'  |single quote             ||103|147|067|01100111|g  |                         |
   |040|050|028|00101000|(  |left/opening parenthesis ||104|150|068|01101000|h  |                         |
   |041|051|029|00101001|   |right/closing parenthesis||105|151|069|01101001|i  |                         |
   |042|052|02A|00101010|*  |asterisk                 ||106|152|06A|01101010|j  |                         |
   |043|053|02B|00101011|+  |plus                     ||107|153|06B|01101011|k  |                         |
   |044|054|02C|00101100|,  |comma                    ||108|154|06C|01101100|l  |                         |
   |045|055|02D|00101101|-  |minus or dash            ||109|155|06D|01101101|m  |                         |
   |046|056|02E|00101110|.  |dot                      ||110|156|06E|01101110|n  |                         |
   |047|057|02F|00101111|/  |forward slash            ||111|157|06F|01101111|o  |                         |
   |048|060|030|00110000|0  |                         ||112|160|070|01110000|p  |                         |
   |049|061|031|00110001|1  |                         ||113|161|071|01110001|q  |                         |
   |050|062|032|00110010|2  |                         ||114|162|072|01110010|r  |                         |
   |051|063|033|00110011|3  |                         ||115|163|073|01110011|s  |                         |
   |052|064|034|00110100|4  |                         ||116|164|074|01110100|t  |                         |
   |053|065|035|00110101|5  |                         ||117|165|075|01110101|u  |                         |
   |054|066|036|00110110|6  |                         ||118|166|076|01110110|v  |                         |
   |055|067|037|00110111|7  |                         ||119|167|077|01110111|w  |                         |
   |056|070|038|00111000|8  |                         ||120|170|078|01111000|x  |                         |
   |057|071|039|00111001|9  |                         ||121|171|079|01111001|y  |                         |
   |058|072|03A|00111010|:  |colon                    ||122|172|07A|01111010|z  |                         |
   |059|073|03B|00111011|;  |semi-colon               ||123|173|07B|01111011|{  |left/opening brace       |
   |060|074|03C|00111100|<  |less than                ||124|174|07C|01111100||  |vertical bar             |
   |061|075|03D|00111101|=  |equal sign               ||125|175|07D|01111101|}  |right/closing brace      |
   |062|076|03E|00111110|>  |greater than             ||126|176|07E|01111110|~  |tilde                    |
   |063|077|03F|00111111|?  |question mark            ||127|177|07F|01111111|DEL|delete                   |
   +---+---+---+--------+---+-------------------------++---+---+---+--------+---+-------------------------+

   ========================================================================== *)

   ASCII_CHARS : packed array [#0..#127] of String[3] =
      (
         {      xx0    xx1    xx2    xx3    xx4    xx5    xx6    xx7   }
         {     -----  -----  -----  -----  -----  -----  -----  -----  }
         {00x} 'NUL', 'SOH', 'STX', 'ETX', 'EOT', 'ENQ', 'ACK', 'BEL',
         {01x} 'BS' , 'HT' , 'LF' , 'VT' , 'FF' , 'CR' , 'SO' , 'SI' ,
         {02x} 'DLE', 'DC1', 'DC2', 'DC3', 'DC4', 'NAK', 'SYN', 'ETB',
         {03x} 'CAN', 'EM' , 'SUB', 'ESC', 'FS' , 'GS' , 'RS' , 'US' ,
         {04x} 'SP' ,'!'   , '"'  , '#'  , '$'  , '%'  , '&'  , '''' ,
         {05x} '('  , ')'  , '*'  , '+'  , ','  , '-'  , '.'  , '/'  ,
         {06x} '0'  , '1'  , '2'  , '3'  , '4'  , '5'  , '6'  , '7'  ,
         {07x} '8'  , '9'  , ':'  , ';'  , '<'  , '='  , '>'  , '?'  ,
         {10x} '@'  , 'A'  , 'B'  , 'C'  , 'D'  , 'E'  , 'F'  , 'G'  ,
         {11x} 'H'  , 'I'  , 'J'  , 'K'  , 'L'  , 'M'  , 'N'  , 'O'  ,
         {12x} 'P'  , 'Q'  , 'R'  , 'S'  , 'T'  , 'U'  , 'V'  , 'W'  ,
         {13x} 'X'  , 'Y'  , 'Z'  , '['  , '\'  , ']'  , '^'  , '_'  ,
         {14x} '`'  , 'a'  , 'b'  , 'c'  , 'd'  , 'e'  , 'f'  , 'g'  ,
         {15x} 'h'  , 'i'  , 'j'  , 'k'  , 'l'  , 'm'  , 'n'  , 'o'  ,
         {16x} 'p'  , 'q'  , 'r'  , 's'  , 't'  , 'u'  , 'v'  , 'w'  ,
         {17x} 'x'  , 'y'  , 'z'  , '{'  , '|'  , '}'  , '~'  , 'DEL'
         {     ----- ----- ----- ----- ----- ----- ----- ----- -----   }
      );

   HEX_CHARS : packed array [0..15] of Char
      = '0123456789ABCDEF';

   ESCAPE_CHARS : set of Char
      = ['A'..'Z','a'..'z','0'..'9','@','*','-','_','+','.','/'];

   FILESYSTEM_SPECIAL_CHARS : set of Char
      = ['\','/',':',  // File/path delimiters   .
         '*','?',      // Wildcards              .
         '<','>','|']; // Redirection            .

// -----------------------------------------------------------------------------               .

const
   MAX_CARDINAL = High(Cardinal);

// -----------------------------------------------------------------------------               .

type
   PPoint2 = ^TPoint2;
   TPoint2 = packed record             // Vgl. Windows.pas: PPoint, TPoint                     .
      Left, Top : Integer;
   end;

   PRect2 = ^TRect2;
   TRect2 = packed record              // Vgl. Windows.pas: PRect, TRect                       .
      Left, Top     : Integer;
      Width, Height : Integer;
   end;

   PSpawnResult = ^TSpawnResult;
   TSpawnResult = packed record
      srResult      : Integer;
      srExitCode    : Cardinal;
      srShortResult : Boolean;         // <-- 'Kurzfassung': ja oder nein ..                   .
   end;

   PWin32VersionResult = ^TWin32VersionResult;
   TWin32VersionResult = packed record
      w32ComposedName : String;
      w32Name         : String;
      w32CSDName      : String;
      w32MajorVersion : Byte;
      w32MinorVersion : Byte;
      w32BuildNumber  : Word;
   end;

// -----------------------------------------------------------------------------               .

type
   LongWordRec = packed record         // Vgl. SysUtils.pas: WordRec, LongRec etc.             .
      LoLo, LoHi, HiLo, HiHi: Byte;
   end;

// ----------------------------------------------------------------------------                .

type
   TSieveResultArray = packed array of Byte;

// -----------------------------------------------------------------------------               .

{$IFDEF CONSOLE}
var
   Error : Text;                       // vgl.: System.Input (STD_INPUT) und                   .
                                       //       System.Output (STD_OUTPUT)                     .
{$ENDIF}

// -----------------------------------------------------------------------------               .

function appGetApplicationFileName ({keine Parameter}): String;
function appGetApplicationPath ({keine Parameter}): String;
function appGetWorkDirectory ({keine Parameter}): String;
function clpbrdCopyTextToClipboard (const text: String): Boolean;
function clpbrdPasteTextFromClipboard ({keine Parameter}): String;
function fmt1ByteToHex (const byte_value: Byte): String; overload;
function fmt1ByteToHex (const byte_value: Byte; const display_prefix: String): String; overload;
function fmt2ByteToHex (const word_value: Word): String; overload;
function fmt2ByteToHex (const word_value: Word; const display_prefix: String): String; overload;
function fmt4ByteToHex (const long_value: Cardinal): String; overload;
function fmt4ByteToHex (const long_value: Cardinal; const display_prefix: String): String; overload;
{$IFDEF DelphiPascalCompiler}
function fmtFormatMessageFromSystem (error_code: DWORD = MAX_CARDINAL; const rem_CRLF: Boolean = True): String;
{$ELSE} {$IFDEF FreePascalCompiler}
function fmtFormatMessageFromSystem (error_code: DWORD; const rem_CRLF: Boolean): String;
{$ENDIF} {$ENDIF}
function fmtFormatNumber (const number: Extended; const digits: Integer): String; overload;
function fmtFormatNumber (const number: Integer; const digits: Integer): String; overload;
function guiClearMsgQueue ({keine Parameter}): Integer;
function guiPoint2 (const p_left, p_top: Integer): TPoint2;
function guiRect2 (const r_left, r_top, r_width, r_height: Integer): TRect2;
function guiCenterFormOnForm (const ref_form, self_form: PRect2; const self_offset: TPoint2): TPoint2;
function guiCenterFormOnScreen (const self_form: PRect2; const self_offset: TPoint2): TPoint2;
function guiGetScreenWidth ({keine Parameter}): Integer;
function guiGetScreenHeight ({keine Parameter}): Integer;
function guiEnableCloseButton (const handle: THandle; const state: Boolean): DWORD;
function guiEnableCloseSysMenu (const handle: THandle; const state: Boolean): Boolean;
function helperSafeDeleteFile (const file_name: String): Boolean;
function helperQuoteFileName (const plain_string: String): String;
function helperDeQuoteFileName (const quoted_string: String): String;
function helperQuoteString (const plain_string: String): String;
function helperDeQuoteString (const quoted_string: String): String;
function resGetVersionEntry (const image_file, version_entry: String): String; overload;
function resGetVersionEntry (const version_entry: String): String; overload;
function resGetXCopyrightInfo ({keine Parameter}): String; overload;
function resGetXCopyrightInfo (const image_file: String): String; overload;
function resGetXDateTimeInfo ({keine Parameter}): String; overload;
function resGetXDateTimeInfo (const image_file: String): String; overload;
function resGetXHistoryInfo ({keine Parameter}): String; overload;
function resGetXHistoryInfo (const image_file: String): String; overload;
function resGetXReleaseInfo ({keine Parameter}): String; overload;
function resGetXReleaseInfo (const image_file: String): String; overload;
function resGetXVersionInfo ({keine Parameter}): String; overload;
function resGetXVersionInfo (const image_file: String): String; overload;
function resGetXWersionInfo ({keine Parameter}): String; overload;
function resGetXWersionInfo (const image_file: String): String; overload;
function strDecrypt (const encrypted_string: String; salt: Word): String;
function strEncrypt (const plain_string: String; salt: Word): String;
{$IFDEF DelphiPascalCompiler}
function strEncryptAndJsEscape (const plain_string: String; salt: Word; const as_widechar: Boolean = False): String;
function strJsEscape (const plain_string: String; var str_result: String; const as_widechar: Boolean = False): Integer; overload;
function strJsEscape (const plain_string: String; const as_widechar: Boolean = False): String; overload;
{$ELSE} {$IFDEF FreePascalCompiler}
function strEncryptAndJsEscape (const plain_string: String; salt: Word; const as_widechar: Boolean): String;
function strJsEscape (const plain_string: String; var str_result: String; const as_widechar: Boolean): Integer; overload;
function strJsEscape (const plain_string: String; const as_widechar: Boolean): String; overload;
{$ENDIF} {$ENDIF}
function strJsUnEscape (const encrypted_string: String; var str_result: String): Integer; overload;
function strJsUnEscape (const encrypted_string: String): String; overload;
function strJsUnEscapeAndDecrypt (const encrypted_string: String; salt: Word): String;
function sysGetFileDate (const file_name: String): TDateTime;
function sysGetFileTime (const file_name: String): TDateTime;
function sysGetFileSize (const file_name: String): Integer;
function sysGetSystemDirectory ({keine Parameter}): String;
function sysGetTempDirectory ({keine Parameter}): String;
function sysGetTempFilename ({keine Parameter}): String;
function sysGetWin32ComputerName ({keine Parameter}): String;
function sysGetWin32UserName ({keine Parameter}): String;
function sysGetWin32Version ({keine Parameter}): TWin32VersionResult;
function sysGetWindowsDirectory ({keine Parameter}): String;
function sysShowErrMsg (const error_message: String): Integer;
function sysSpawnProcess (const image_file, run_parameters, work_directory: String): TSpawnResult;
function sysSpawnProcessAndWait (const image_file, run_parameters, work_directory: String): TSpawnResult;
function sysStartMailClient (const mail_to, mail_subject: String; const mail_body: String; const mail_attachments: array of String): Boolean;
{$IFDEF DelphiPascalCompiler}
function timeConvertLocalTimeToUTC (const datetime: TDateTime; const NULL_onerror: Boolean = False): TDateTime;
function timeConvertUTCToLocalTime (const datetime: TDateTime; const NULL_onerror: Boolean = False): TDateTime;
{$ELSE} {$IFDEF FreePascalCompiler}
function timeConvertLocalTimeToUTC (const datetime: TDateTime; const NULL_onerror: Boolean): TDateTime;
function timeConvertUTCToLocalTime (const datetime: TDateTime; const NULL_onerror: Boolean): TDateTime;
{$ENDIF} {$ENDIF}
function timeConvertLocalTimeToSwatchBEATs (const datetime: TDateTime): Word;
function timeConvertSwatchBEATsToLocalTime (const beats: Smallint): TDateTime;

// ----------------------------------------------------------------------------                .

function Sieve_Of_Eratosthenes (const N: Integer; var A: TSieveResultArray): Integer;

// ----------------------------------------------------------------------------                .

// ---------------                                                                             .
   implementation
// ---------------                                                                             .

(*
 * grep -n -i "Assert" -A 1 Misc_Utilities.pas | unix2dos > Misc_Utilities.assert
 *)

uses
{$IFDEF DelphiPascalCompiler}
   SysUtils,
   ShellAPI,
   Messages;
{$ELSE} {$IFDEF FreePascalCompiler}
   MissingInFPC,
   SysUtils;
{$ENDIF} {$ENDIF}

// ----------------------------------------------------------------------------                .
function appGetApplicationFileName (): String;
begin
   Result := ExtractFileName(ParamStr(0));
end;

// ----------------------------------------------------------------------------                .
function appGetApplicationPath (): String;
begin
   Result := ExtractFilePath(ParamStr(0));
   if ((Result <> '') and
       (Result[Length(Result)] <> '\')) then
      Result := (Result + '\');
end;

// ----------------------------------------------------------------------------                .
function appGetWorkDirectory ({keine Parameter}): String;
begin
   Result := GetCurrentDir();
   if ((Result <> '') and
       (Result[Length(Result)] <> '\')) then
      Result := (Result + '\');
end;

// ----------------------------------------------------------------------------                .
// From the MS Win32-API help file:                                                            .
//                                                                                             .
// HANDLE SetClipboardData(UINT uFormat,  .. clipboard format                                  .
//                         HANDLE hMem);  .. data handle                                       .
//                                                                                             .
// Once SetClipboardData is called, the system owns the object identified by the hMem          .
// parameter. The application can read the data, but must not free the handle or leave it      .
// locked. If the hMem parameter identifies a memory object, the object must have been         .
// allocated using the GlobalAlloc function with the GMEM_MOVEABLE and GMEM_DDESHARE flags.    .
// ----------------------------------------------------------------------------                .
function clpbrdCopyTextToClipboard (const text: String): Boolean;
var
   hglbCopy: HGLOBAL;
   lptstrCopy: POINTER;
begin
   Result := False;
   if (OpenClipboard(0)) then begin
      if (EmptyClipboard()) then begin
         hglbCopy := GlobalAlloc((GMEM_MOVEABLE or GMEM_DDESHARE),(Length(text) + 1));
         if (hglbCopy <> 0) then begin
            lptstrCopy := GlobalLock(hglbCopy);
            if (Assigned(lptstrCopy)) then begin
               Move(PChar(text)^,lptstrCopy^,Length(text));
               PByteArray(lptstrCopy)^[Length(text)] := Ord(#0);
               Result := (SetClipboardData(CF_TEXT,hglbCopy) <> 0);
               GlobalUnlock(hglbCopy);
            end; // GlobalLock() fehlgeschlagen                                                .
          { GlobalFree(hglbCopy); <- NEIN !! -- siehe Kommentar ... }
         end; // GlobalAlloc() fehlgeschlagen                                                  .
      end; // EmptyClipBoard() fehlgeschlagen                                                  .
      CloseClipboard();
   end; // OpenClipBoard() fehlgeschlagen                                                      .
end;

// ----------------------------------------------------------------------------                .
function clpbrdPasteTextFromClipboard (): String;
var
   hglbCopy: HGLOBAL;
   lptstrSize: DWORD;
   lptstrCopy: POINTER;
begin
   Result := '';
   if (OpenClipboard(0)) then begin
      hglbCopy := GetClipboardData(CF_TEXT);
      if (hglbCopy <> 0) then begin
         lptstrSize := GlobalSize(hglbCopy);
         if (lptstrSize <> 0) then begin
            lptstrCopy := GlobalLock(hglbCopy);
            if (Assigned(lptstrCopy)) then begin
               SetLength(Result,lptstrSize);
               Move(lptstrCopy^,PChar(Result)^,lptstrSize);
               GlobalUnlock(hglbCopy);
            end; // GlobalLock() fehlgeschlagen                                                .
         end; // GlobalSize() fehlgeschlagen                                                   .
      end; // GetClipboardData() fehlgeschlagen oder Inhalt <> CF_TEXT                         .
      CloseClipboard();
   end; // OpenClipBoard() fehlgeschlagen                                                      .
end;

// ----------------------------------------------------------------------------                .
// Konvertiert <byte_value> in einen Hex-String der Lnge 2                                    .
// ----------------------------------------------------------------------------                .
function fmt1ByteToHex (const byte_value: Byte): String;
begin
   Result := (HEX_CHARS[((byte_value shr 4) and $0F)] +
              HEX_CHARS[( byte_value        and $0F)]);
end;

// ----------------------------------------------------------------------------                .
function fmt1ByteToHex (const byte_value: Byte; const display_prefix: String): String;
begin
   if (display_prefix <> '') then begin
      Result := (display_prefix + fmt1ByteToHex(byte_value));
   end else begin
   {$IFDEF DelphiPascalCompiler}
      Result := (HexDisplayPrefix + fmt1ByteToHex(byte_value));
   {$ELSE} {$IFDEF FreePascalCompiler}
      Result := ('$' + fmt1ByteToHex(byte_value));
   {$ENDIF} {$ENDIF}
   end;
end;

// ----------------------------------------------------------------------------                .
// Konvertiert <word_value> in einen Hex-String der Lnge 4                                    .
// ----------------------------------------------------------------------------                .
function fmt2ByteToHex (const word_value: Word): String;
begin
   Result := (fmt1ByteToHex(WordRec(word_value).Hi) + // Hi(), Lo() sind Funktionen,           .
              fmt1ByteToHex(WordRec(word_value).Lo)); // und somit langsamer als ein Cast      .
end;

// ----------------------------------------------------------------------------                .
function fmt2ByteToHex (const word_value: Word; const display_prefix: String): String;
begin
   if (display_prefix <> '') then begin
      Result := (display_prefix + fmt2ByteToHex(word_value));
   end else begin
   {$IFDEF DelphiPascalCompiler}
      Result := (HexDisplayPrefix + fmt2ByteToHex(word_value));
   {$ELSE} {$IFDEF FreePascalCompiler}
      Result := ('$' + fmt2ByteToHex(word_value));
   {$ENDIF} {$ENDIF}
   end;
end;

// ----------------------------------------------------------------------------                .
// Konvertiert <long_value> in einen Hex-String der Lnge 8                                    .
// ----------------------------------------------------------------------------                .
function fmt4ByteToHex (const long_value: Cardinal): String;
begin
   Result := (fmt2ByteToHex(LongRec(long_value).Hi) + // HiWord(), LoWord() sind Funktionen,   .
              fmt2ByteToHex(LongRec(long_value).Lo)); // und somit langsamer als ein Cast      .
end;

// ----------------------------------------------------------------------------                .
function fmt4ByteToHex (const long_value: Cardinal; const display_prefix: String): String;
begin
   if (display_prefix <> '') then begin
      Result := (display_prefix + fmt4ByteToHex(long_value));
   end else begin
   {$IFDEF DelphiPascalCompiler}
      Result := (HexDisplayPrefix + fmt4ByteToHex(long_value));
   {$ELSE} {$IFDEF FreePascalCompiler}
      Result := ('$' + fmt4ByteToHex(long_value));
   {$ENDIF} {$ENDIF}
   end;
end;

// ----------------------------------------------------------------------------                .
{$IFDEF DelphiPascalCompiler}
function fmtFormatMessageFromSystem (error_code: DWORD = MAX_CARDINAL; const rem_CRLF: Boolean = True): String;
{$ELSE} {$IFDEF FreePascalCompiler}
function fmtFormatMessageFromSystem (error_code: DWORD; const rem_CRLF: Boolean): String;
{$ENDIF} {$ENDIF}
const
   MAX_BUF = (1024 * 8);    // 8 kB sollten gengen :)                                         .
var
   pBuf: packed array [0..MAX_BUF] of Char;
begin
   // Kein Wert angegeben? -> Wert aus GetLastError() ermitteln.                               .
   if (error_code = MAX_CARDINAL) then begin
      error_code := GetLastError();
   // Fehler-Code vom System in einen String bersetzen lassen.                                .
   end;
   if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
                     NIL,
                     error_code,
                     ((SUBLANG_DEFAULT shl 10) + LANG_NEUTRAL),
                     @pBuf,
                     (MAX_BUF + 1),
                     NIL) = 0) then begin
      Result := '';                  // Funktion fehlgeschlagen :-(                            .
   end else begin
      if (rem_CRLF) then begin       // Funktion erfolgreich                                   .
         Result := Trim(StrPas(pBuf));
      end else begin                 // 'Result' kann mit CRLF abgeschlossen sein; je nach     .
         Result := StrPas(pBuf);     // Flag entfernen oder belassen ...                       .
      end;
   end;
end;

// ----------------------------------------------------------------------------                .
// Formatiert eine REELE Zahl mit <digits> Nachkommastellen und Tausenderpunkt, eg:            .
//    (47,0) = '47'; (47,2) = '47,00'; (4711,0) = '4.711'; (4711,2) = '4.711,00';              .
// ----------------------------------------------------------------------------                .
function fmtFormatNumber (const number: Extended; const digits: Integer): String;
begin
   Assert( ((digits >= 0) and (digits < 10)),
           'param <digits> ist nicht im Bereich von [0..9]' );
   Result := Format('%0.*n',[digits,number]);
end;

// ----------------------------------------------------------------------------                .
// Formatiert eine ORDINALE Zahl mit <digits> fhrenden Nullen, eg:                            .
//    (47,2) = '47'; (47,4) = '0047'; (4711,4) = '4711'; (4711,6) = '004711';                  .
// ----------------------------------------------------------------------------                .
function fmtFormatNumber (const number: Integer; const digits: Integer): String;
begin
   Assert( ((digits >= 0) and (digits < 10)),
           'param <digits> ist nicht im Bereich von [0..9]' );
   Result := Format('%*.0d',[digits,number]);
end;

// ----------------------------------------------------------------------------                .
function guiClearMsgQueue (): Integer;
{$IFDEF CONSOLE}
{$ELSE}
var
   msg: TMsg;
{$ENDIF}
begin
{$IFDEF CONSOLE}
   raise Exception.Create('Fatal: can not use "guiClearMsgQueue()" in console mode.')
{$ELSE}
   Result := 0;
   while (PeekMessage(msg,0,0,0,PM_REMOVE)) do begin
      Inc(Result);
   end;
{$ENDIF}
end;

// ----------------------------------------------------------------------------                .
function guiPoint2 (const p_left, p_top: Integer): TPoint2;
begin
   Result.Left := p_left;
   Result.Top := p_top;
end;

// ----------------------------------------------------------------------------                .
function guiRect2 (const r_left, r_top, r_width, r_height: Integer): TRect2;
begin
   Result.Left := r_left;
   Result.Top := r_top;
   Result.Width := r_width;
   Result.Height := r_height;
end;

// ----------------------------------------------------------------------------                .
function __int_CalcCoords (const ref_form: PRect2; self_form: PRect2; const self_offset: TPoint2): TPoint2;
var
   r_self: TRect2;
begin
   r_self := guiRect2(0,0,0,0);
   if (self_form = NIL) then begin
      self_form := @r_self;
   end;
   if (ref_form = NIL) then begin    // kein Referenz-Formular?                             .
      if (self_offset.Left = 0) then // keine Left- Offset-Angabe?:                         .
         Result.Left := ((guiGetScreenWidth()-self_form^.Width) div 2) // Bildschirm-X-Mitte.
      else                           // sonst:                                              .
         Result.Left := self_offset.Left;                  // 'einfach' nur Offset          .
      if (self_offset.Top = 0) then  // keine Top- Offset-Angabe?:                          .
         Result.Top := ((guiGetScreenHeight()-self_form^.Height) div 2) // Bildschirm-Y-Mitte
      else                           // sonst:                                              .
         Result.Top := self_offset.Top;                    // 'einfach' nur Offset          .
   end else begin
      if (self_offset.Left = 0) then // keine Left- Offset-Angabe?:                         .
                                     // Formular-X-Mitte von <ref_form>                     .
         Result.Left := (ref_form^.Left + ((ref_form^.Width-self_form^.Width) div 2))
      else                           // sonst:                                              .
         Result.Left := (ref_form^.Left + self_offset.Left); // <ref_form>.Left + Offset    .
      if (self_offset.Top = 0) then  // keine Top- Offset-Angabe?:                          .
                                     // Formular-Y-Mitte von <ref_from>                     .
         Result.Top :=(ref_form^.Top + ((ref_form^.Height-self_form^.Height) div 2))
      else                           // sonst:                                              .
         Result.Top :=(ref_form^.Top + self_offset.Top); // <ref_form>.Top + Offset         .
   end;
end;

// ----------------------------------------------------------------------------                .
function guiCenterFormOnForm (const ref_form, self_form: PRect2; const self_offset: TPoint2): TPoint2;
begin
   Result := __int_CalcCoords(ref_form,self_form,self_offset);
end;

// ----------------------------------------------------------------------------                .
function guiCenterFormOnScreen (const self_form: PRect2; const self_offset: TPoint2): TPoint2;
begin
   Result := __int_CalcCoords(NIL,self_form,self_offset);
end;

// ----------------------------------------------------------------------------                .
function guiGetScreenWidth (): Integer;
begin
   Result := GetSystemMetrics(SM_CXSCREEN);
end;

// ----------------------------------------------------------------------------                .
function guiGetScreenHeight (): Integer;
begin
   Result := GetSystemMetrics(SM_CYSCREEN);
end;

// ----------------------------------------------------------------------------                .
function guiEnableCloseButton (const handle: THandle; const state: Boolean): DWORD;
begin
{$IFDEF CONSOLE}
   raise Exception.Create('Fatal: can not use "guiEnableCloseButton()" in console mode.')
{$ELSE}
   Assert( (handle <> INVALID_HANDLE_VALUE) ,
           'param <handle> ist nicht gueltig' );
   if (state) then begin
      Result := SetClassLong(handle,GCL_STYLE,(GetClassLong(handle,GCL_STYLE) and not(CS_NOCLOSE)));
   end else begin
      Result := SetClassLong(handle,GCL_STYLE,(GetClassLong(handle,GCL_STYLE) or CS_NOCLOSE));
   end;
{$ENDIF}
end;

// ----------------------------------------------------------------------------                .
function guiEnableCloseSysMenu (const handle: THandle; const state: Boolean): Boolean;
{$IFDEF CONSOLE}
{$ELSE}
var
   sys_menu: THandle;
{$ENDIF}
begin
{$IFDEF CONSOLE}
   raise Exception.Create('Fatal: can not use "guiEnableCloseSysMenu()" in console mode.')
{$ELSE}
   Assert( (handle <> INVALID_HANDLE_VALUE) ,
           'param <handle> ist nicht gueltig' );
   sys_menu := GetSystemMenu(handle,false);
   if (state) then begin
      Result := EnableMenuItem(sys_menu,SC_CLOSE,MF_BYCOMMAND);
   end else begin
      Result := EnableMenuItem(sys_menu,SC_CLOSE,(MF_BYCOMMAND or MF_GRAYED));
   end;
   DrawMenuBar(handle);
{$ENDIF}
end;

// 25.08.'04: Prfung erweitert um: "und attr ist _kein_ Directory"                            .
// ----------------------------------------------------------------------------                .
function helperSafeDeleteFile (const file_name: String): Boolean;
var
   attr: Integer;
begin
   Assert((Trim(file_name) <> '') ,
          'param <file_name> darf nicht leer sein' );
   attr := FileGetAttr(file_name);
   if ((attr <> (-1)) and
       ((attr and faDirectory) <> faDirectory))  then begin
      attr := (attr and not(faReadOnly)   // may not be able to delete ReadOnly - clear bit    .
                    and not(faHidden  )   // may not be able to delete Hidden - clear bit      .
                    and not(faSysFile )); // may not be able to delete System - clear bit      .
      Result := (FileSetAttr(file_name,attr) = 0);
      if (Result) then begin
         Result := DeleteFile(file_name);
      end;
   end else begin
      Result := False;
   end;
end;

// ----------------------------------------------------------------------------                .
function helperQuoteFileName (const plain_string: String): String;
begin
   if (Pos(' ',plain_string) > 0) then begin
      Result := helperQuoteString(plain_string);
   end else begin
      Result := plain_string;
   end;
end;

// ----------------------------------------------------------------------------                .
function helperDeQuoteFileName (const quoted_string: String): String;
begin
   Result := helperDeQuoteString(quoted_string);
end;

// ----------------------------------------------------------------------------                .
function helperQuoteString (const plain_string: String): String;
begin
   Result := QuotedStr(plain_string);
end;

// ----------------------------------------------------------------------------                .
function helperDeQuoteString (const quoted_string: String): String;
begin
   if ((Length(quoted_string) >= 2) and
       (quoted_string[1] in ['"','''']) and
       (quoted_string[Length(quoted_string)] = quoted_string[1])) then begin
      Result := Copy(quoted_string,2,(Length(quoted_string) - 2));
   end else begin
      Result := quoted_string;
   end;
end;

// ----------------------------------------------------------------------------                .
// Interne Funktion; Wrapper fr 2x resGetVersionEntry()                                       .
// ----------------------------------------------------------------------------                .
function __int_GetVersionEntry (const image_file, version_entry: String): String;
var
   data_size: Integer;
   data, data_2: Pointer;
   lang_id : ^LongInt;
   unused: DWord;
begin
   Assert( (Trim(image_file) <> '') ,
           'param <image_file> darf nicht leer sein' );
   Assert( (Trim(version_entry) <> '') ,
           'param <version_entry> darf nicht leer sein' );
   Result := '';
   if (FileExists(image_file)) then begin
      data_size := GetFileVersionInfoSize(PChar(image_file),unused);
      if (data_size > 0) then begin
         GetMem(data,data_size);
         if (GetFileVersionInfo(PChar(image_file),0,data_size,data)) then begin
            // Format: TEXT("\\StringFileInfo\\<Language_ID>\\FileVersion")                    .
            VerQueryValue(data,'\VarFileInfo\Translation',POINTER(lang_id),unused);
            VerQueryValue(data,PChar(Format('\StringFileInfo\%s%s\%s',
                                            [fmt2ByteToHex(LongRec(LONGINT(lang_id^)).Lo),
                                             fmt2ByteToHex(LongRec(LONGINT(lang_id^)).Hi),
                                             version_entry])),
                          data_2,unused);
            FreeMem(data,data_size);
            Result := StrPas(data_2);
         end; // GetFileVersionInfo() fehlgeschlagen                                           .
      end; // GetFileVersionInfoSize() fehlgeschlagen                                          .
   end; // FileExists() fehlgeschlagen                                                         .
end;

// ----------------------------------------------------------------------------                .
function resGetVersionEntry (const image_file, version_entry: String): String;
begin
   Assert( (Trim(image_file) <> '') ,
           'param <image_file> darf nicht leer sein' );
   Result := __int_GetVersionEntry(image_file,version_entry);
end;

// ----------------------------------------------------------------------------                .
function resGetVersionEntry (const version_entry: String): String;
begin
   Result := __int_GetVersionEntry(ParamStr(0),version_entry);
end;

// ----------------------------------------------------------------------------                .
function resGetXCopyrightInfo (): String;
begin
   Result := resGetXCopyrightInfo(ParamStr(0));
end;

// ----------------------------------------------------------------------------                .
function resGetXCopyrightInfo (const image_file: String): String;
begin
   Assert( (Trim(image_file) <> '') ,
           'param <image_file> darf nicht leer sein' );
   Result := resGetVersionEntry(image_file,'X_Copyright_Info');
end;

// ----------------------------------------------------------------------------                .
function resGetXDateTimeInfo (): String;
begin
   Result := resGetXDateTimeInfo(ParamStr(0));
end;

// ----------------------------------------------------------------------------                .
function resGetXDateTimeInfo (const image_file: String): String;
begin
   Assert( (Trim(image_file) <> '') ,
           'param <image_file> darf nicht leer sein' );
   Result := resGetVersionEntry(image_file,'X_DateTime_Info');
end;

// ----------------------------------------------------------------------------                .
// Interne Funktion; Wrapper fr 2x resGetXHistoryInfo()                                       .
// ----------------------------------------------------------------------------                .
function __int_LoadRCDataResource (const hModule: HINST; const lpName: PChar): PChar;
var
   r: HRSRC;
   rs: DWORD;
   rh: HGLOBAL;
   p: PChar;
begin
   Result := NIL;
   r := FindResource(hModule,lpName,RT_RCDATA);
   if (r <> 0) then begin
      rs := SizeOfResource(hModule,r);
      if (rs <> 0) then begin
         rh := LoadResource(hModule,r);
         if (rh <> 0) then begin
            p := LockResource(rh);
            if (p <> NIL) then begin
               Result := StrAlloc(rs);
               Move(p^,Result^,rs);
               UnLockResource(rh);
            end; // LockResource() fehlgeschlagen                                              .
            FreeResource(rh);
         end; // LoadResource() fehlgeschlagen                                                 .
      end; // SizeOfResource() fehlgeschlagen                                                  .
   end; // FindResource() fehlgeschlagen                                                       .
end;

// ----------------------------------------------------------------------------                .
function resGetXHistoryInfo (): String;
begin
   Result := StrPas(__int_LoadRCDataResource(0,'X_History_Info'));
end;

// ----------------------------------------------------------------------------                .
function resGetXHistoryInfo (const image_file: String): String;
var
   hi: HINST;
begin
   Assert( (Trim(image_file) <> '') ,
           'param <image_file> darf nicht leer sein' );
   Result := '';
   if (FileExists(image_file)) then begin
   {$IFDEF DelphiPascalCompiler}
      hi := SafeLoadLibrary(image_file);
   {$ELSE} {$IFDEF FreePascalCompiler}
      hi := SafeLoadLibrary(image_file,SEM_NOOPENFILEERRORBOX);
   {$ENDIF} {$ENDIF}
      if (hi <> 0) then begin
         Result := StrPas(__int_LoadRCDataResource(hi,'X_History_Info'));
         FreeLibrary(hi);
      end; // SafeLoadLibrary() failed                                                         .
   end; // FileExists() failed                                                                 .
end;

// ----------------------------------------------------------------------------                .
function resGetXReleaseInfo (): String;
begin
   Result := resGetXReleaseInfo(ParamStr(0));
end;

// ----------------------------------------------------------------------------                .
function resGetXReleaseInfo (const image_file: String): String;
begin
   Assert( (Trim(image_file) <> '') ,
           'param <image_file> darf nicht leer sein' );
   Result := resGetVersionEntry(image_file,'X_Release_Info');
end;

// ----------------------------------------------------------------------------                .
function resGetXVersionInfo (): String;
begin
   Result := resGetXVersionInfo(ParamStr(0));
end;

// ----------------------------------------------------------------------------                .
function resGetXVersionInfo (const image_file: String): String;
begin
   Assert( (Trim(image_file) <> '') ,
           'param <image_file> darf nicht leer sein' );
   Result := resGetVersionEntry(image_file,'X_Version_Info');
end;

// ----------------------------------------------------------------------------                .
function resGetXWersionInfo (): String;
begin
   Result := resGetXWersionInfo(ParamStr(0));
end;

// ----------------------------------------------------------------------------                .
function resGetXWersionInfo (const image_file: String): String;
begin
   Assert( (Trim(image_file) <> '') ,
           'param <image_file> darf nicht leer sein' );
   Result := resGetVersionEntry(image_file,'X_Wersion_Info');
end;

// -----------------------------------------------------------------------------               .
const
   C1 = 52845; // To-Check: can C1 % C2 be any number ?!? -- Yes ...                           .
   C2 = 22719;

// ----------------------------------------------------------------------------                .
// Decrypt <encrypted_string> using C1 & C2, modifying the algo with <salt>                    .
// -----------------------------------------------------------------------------               .
function strDecrypt (const encrypted_string: String; salt: Word): String;
var
   i: Integer;
begin
   if (encrypted_string <> '') then begin
      SetLength(Result,Length(encrypted_string));
      for i := 1 to Length(encrypted_string) do begin
         Result[i] := Chr( Ord(encrypted_string[i]) xor Byte(salt shr 8) );
      {$IFDEF DelphiPascalCompiler}
         salt := LongRec(((Integer(Word(Ord(encrypted_string[i])) + salt) and $00007FFF) * C1) + C2).Lo;
      {$ELSE} {$IFDEF FreePascalCompiler}
         salt := LoWord(((Integer(Word(Ord(encrypted_string[i])) + salt) and $00007FFF) * C1) + C2);
      {$ENDIF} {$ENDIF}
      end;
   end else begin
      Result := '';
   end;
end;

{
// -----------------------------------------------------------------------------               .
function strDecrypt (const C1,C2: Word; const encrypted_string: String; salt: Word): String;
begin
   Result := strDecrypt(encrypted_string,salt);
end;
}

// ----------------------------------------------------------------------------                .
// Encrypt <plain_string> using C1 & C2, modifying the algo with <salt>                        .
// -----------------------------------------------------------------------------               .
function strEncrypt (const plain_string: String; salt: Word): String;
var
   i: Integer;
begin
   if (plain_string <> '') then begin
      SetLength(Result,Length(plain_string));
      for i := 1 to Length(plain_string) do begin
         Result[i] := Chr( Ord(plain_string[i]) xor Byte(salt shr 8) );
      {$IFDEF DelphiPascalCompiler}
         salt := LongRec(((Integer(Word(Ord(Result[i])) + salt) and $00007FFF) * C1) + C2).Lo;
      {$ELSE} {$IFDEF FreePascalCompiler}
         salt := LoWord(((Integer(Word(Ord(Result[i])) + salt) and $00007FFF) * C1) + C2);
      {$ENDIF} {$ENDIF}
      end;
   end else begin
      Result := '';
   end;
end;

{
// -----------------------------------------------------------------------------               .
function strEncrypt (const C1,C2: Word; const plain_string: String; salt: Word): String;
begin
   Result := strEncrypt(plain_string,salt);
end;
}

// ----------------------------------------------------------------------------                .
{$IFDEF DelphiPascalCompiler}
function strEncryptAndJsEscape (const plain_string: String; salt: Word; const as_widechar: Boolean = False): String;
{$ELSE} {$IFDEF FreePascalCompiler}
function strEncryptAndJsEscape (const plain_string: String; salt: Word; const as_widechar: Boolean): String;
{$ENDIF} {$ENDIF}
begin
   Assert( (plain_string <> ''),                          // *NICHT* Trim(), da ein Escape von .
           'param <plain_string> darf nicht leer sein' );   // ' ' durch aus legal ist!        .
   Result := strJsEscape(strEncrypt(plain_string,salt),as_widechar);
end;

// ----------------------------------------------------------------------------                .
// JavaScript Escape() Funktion                                                                .
//                                                                                             .
// From the documentation at <http://www.devguru.com>:                                         .
//                                                                                             .
// The top-level function 'escape' encodes the string that is contained in the string argument .
// to make it portable. A string is considered portable if it can be transmitted across any    .
// network to any computer that supports ASCII characters. To make a string portable, char-    .
// acters other than the following 69 ASCII characters must be encoded:                        .
//                                                                                             .
// ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890@*-_+./                       .
//                                                                                             .
// All other characters are converted either to their two digit (%xx) or four digit (%uxxxx)   .
// hexadecimal equivalent (refered to as the character's "hexadecimal escape sequence"). For   .
// example, a blank space will be represented by %20 and a semicolon by %3B.                   .
//                                                                                             .
//    <plain_string> .. String to escape                                                       .
//    <str_result>   .. VAR reference to store result                                          .
//    <as_widechar>  .. Flag; False to use 2 byte coding; True to use 4 Byte coding            .
//    Result         .. = 0 on success; > 0 on failure (= error-position in input string)      .
// ----------------------------------------------------------------------------                .
{$IFDEF DelphiPascalCompiler}
function strJsEscape (const plain_string: String; var str_result: String; const as_widechar: Boolean = False): Integer;
{$ELSE} {$IFDEF FreePascalCompiler}
function strJsEscape (const plain_string: String; var str_result: String; const as_widechar: Boolean): Integer;
{$ENDIF} {$ENDIF}
var
   i: Integer;
begin
   Assert( (plain_string <> ''),                          // *NICHT* Trim(), da ein Escape von .
           'param <plain_string> darf nicht leer sein' );   // ' ' durch aus legal ist!        .
   // Initialisierung                                                                          .
   Result := 0;
   str_result := '';
   i := 1;
   // ber den gesamten String iterieren:                                                      .
   while (i <= Length(plain_string)) do begin
      if (plain_string[i] in ESCAPE_CHARS) then begin
         str_result := (str_result + plain_string[i]);
      end else begin
         if (as_widechar) then begin
            str_result := (str_result + '%' + 'u' + fmt2ByteToHex(Word(Ord(plain_string[i]))));
         end else begin
            str_result := (str_result + '%' + fmt1ByteToHex(Ord(plain_string[i])));
         end;
      end;
      Inc(i);
   end;
end;

// ----------------------------------------------------------------------------                .
{$IFDEF DelphiPascalCompiler}
function strJsEscape (const plain_string: String; const as_widechar: Boolean = False): String;
{$ELSE} {$IFDEF FreePascalCompiler}
function strJsEscape (const plain_string: String; const as_widechar: Boolean): String;
{$ENDIF} {$ENDIF}
var
   s: String;
begin
   Assert( (plain_string <> ''),                          // *NICHT* Trim(), da ein Escape von .
           'param <plain_string> darf nicht leer sein' );   // ' ' durch aus legal ist!        .
   if (strJsEscape(plain_string,s,as_widechar) = 0) then begin
      Result := s;
   end else begin
      Result := plain_string;
   end;
end;

// ----------------------------------------------------------------------------                .
// JavaScript UnEscape() Funktion                                                              .
// ----------------------------------------------------------------------------                .
function strJsUnEscape (const encrypted_string: String; var str_result: String): Integer;
var
   i, c_idx: Integer;
   error: Boolean;
begin
   Assert( (Trim(encrypted_string) <> ''),
           'param <encrypted_string> darf nicht leer sein' );
   // Initialisierung                                                                          .
   Result := 0;
   str_result := '';
   i := 1;
   error := False;
   // ber den gesamten String iterieren:                                                      .
   while (i <= Length(encrypted_string)) do begin
      // s[i] ist '%' -- nachfolgend erwartet wird '%' oder '$$' oder 'u$$$$'                  .
      if (encrypted_string[i] = '%') then begin
         // Fehler, '%' kann nicht alleine stehen                                              .
         if (i = Length(encrypted_string)) then begin
            error := True;
         // (streng genommen: nicht ganz legal ;-) '%' ist gestuffed -- beide konsumieren      .
        end else if (encrypted_string[i + 1] = '%') then begin
            Inc(i,2);                          // 2 Zeichen konsumieren ('%'+'%')              .
         // Ein fhrendes 'u' bedeutet: 'Langform', d.h. Hex-Digit ist 4-stellig               .
        end else if (encrypted_string[i + 1] = 'u') then begin
            if ((i + 1 + 4) <= Length(encrypted_string)) then begin // <-- '%'+'u'+'$$$$'      .
            {$IFDEF DelphiPascalCompiler}
               c_idx := StrToIntDef((HexDisplayPrefix + Copy(encrypted_string,(i + 1 + 1),4)),(-1));
            {$ELSE} {$IFDEF FreePascalCompiler}
               c_idx := StrToIntDef(('$' + Copy(encrypted_string,(i + 1 + 1),4)),(-1));
            {$ENDIF} {$ENDIF}
               if (c_idx <> (-1)) then begin
                  str_result := (str_result + Chr(c_idx)); // Gltige Hex-Zahl -> Char         .
                  Inc(i,(1 + 1 + 4));          // 6 Zeichen kosumieren ('%'+'u'+'$$$$')        .
               end else begin
                  error := True; // Fehler: die 4 Zeichen waren keine Hex-Zahl                 .
               end;
            end else begin
               error := True; // Fehler: es sind weniger als (1+4) Zeichen brig               .
            end;
         // Dieser Fall bleibt ber: 'Kurzform', d.h. Hex-Digit ist 2-stellig                  .
         end else begin
            if ((i + 2) <= Length(encrypted_string)) then begin   // <-- '%'+'$$'              .
            {$IFDEF DelphiPascalCompiler}
               c_idx := StrToIntDef((HexDisplayPrefix + Copy(encrypted_string,(i + 1),2)),(-1));
            {$ELSE} {$IFDEF FreePascalCompiler}
               c_idx := StrToIntDef(('$' + Copy(encrypted_string,(i + 1),2)),(-1));
            {$ENDIF} {$ENDIF}
               if (c_idx <> (-1)) then begin
                  str_result := (str_result + Chr(c_idx)); // Gltige Hex-Zahl -> Char         .
                  Inc(i,(1 + 2));              // 3 Zeichen konsumieren ('%'+'$$')             .
               end else begin
                  error := True; // Fehler: die 2 Zeichen waren keine Hex-Zahl                 .
               end;
            end else begin
               error := True; // Fehler: es sind weniger als 2 Zeichen brig                   .
            end;
         end;
      // s[i] ist _nicht_ '%' -- erwartet wird ein legales Zeichen aus <EscapeChars>           .
      end else if (encrypted_string[i] in ESCAPE_CHARS) then begin
         str_result := (str_result + encrypted_string[i]);
         Inc(i,1);
      // s[i] ist weder '%' noch ein legales Zeichen aus <EscapeChars> -> Fehler               .
      end else begin
         error := True;
      end;
      if (error) then begin
         Result := i;   // <-- String-Position der fehlerhaften Stelle retour                  .
         Break;
      end;
   end;
end;

// ----------------------------------------------------------------------------                .
function strJsUnEscape (const encrypted_string: String): String;
var
   s: String;
begin
   Assert( (Trim(encrypted_string) <> ''),
           'param <encrypted_string> darf nicht leer sein' );
   if (strJsUnEscape(encrypted_string,s) = 0) then begin
      Result := s;
   end else begin
      Result := encrypted_string;
   end;
end;

// ----------------------------------------------------------------------------                .
// used for "easy" string encryption/ decryption                                               .
// ----------------------------------------------------------------------------                .
function strJsUnEscapeAndDecrypt (const encrypted_string: String; salt: Word): String;
begin
   Assert( (Trim(encrypted_string) <> ''),
           'param <encrypted_string> darf nicht leer sein' );
   Result := strDecrypt(strJsUnEscape(encrypted_string),salt);
end;

// ----------------------------------------------------------------------------                .
// 'share-aware' __int_GetFileInfo():                                                          .
//    Instead of open/ determine/ close (wich may fail if the file is opened                   .
//    by another process) the FindFirst()/ FindNext() API is used.                             .
// ----------------------------------------------------------------------------                .
function __int_GetFileInfo (const file_name: String; var search_rec: TSearchRec): Boolean;
begin
   Assert( (Trim(file_name) <> ''),
           'param <file_name> darf nicht leer sein' );
   Result := False;
   if (FindFirst(file_name,faAnyFile,search_rec) = 0) then begin
      Result := True;          // Lookup war 1x erfolgreich: Ergebnis bisher OK ..             .
                               // Lookup nochmals erfolgreich? (d.h. file_name mehrdeutig?     .
      if (FindNext(search_rec) = 0) then begin
         Result := False;      // Wahrscheinlich Wildecard angegeben, doch nicht OK ..         .
      end;
   end;
   FindClose(search_rec);
end;

// ----------------------------------------------------------------------------                .
function sysGetFileDate (const file_name: String): TDateTime;
var
   sr: TSearchRec;
begin
   Assert( (Trim(file_name) <> ''),
           'param <file_name> darf nicht leer sein' );
   Result := 0.0;
   if (__int_GetFileInfo(file_name,sr)) then begin
   {$IFDEF DelphiPascalCompiler}
      Result := Int(FileDateToDateTime(sr.Time)); // _nur_ Datum, Uhrzeit = 00:00              .
   {$ELSE} {$IFDEF FreePascalCompiler}
      Result := Trunc(FileDateToDateTime(sr.Time)); // _nur_ Datum, Uhrzeit = 00:00            .
   {$ENDIF} {$ENDIF}
   end;
end;

// ----------------------------------------------------------------------------                .
function sysGetFileTime (const file_name: String): TDateTime;
var
   sr: TSearchRec;
begin
   Assert( (Trim(file_name) <> ''),
           'param <file_name> darf nicht leer sein' );
   Result := 0.0;
   if (__int_GetFileInfo(file_name,sr)) then begin
      Result := Frac(FileDateToDateTime(sr.Time)); // _nur_ Uhrzeit, Datum = 30.12.1899        .
   end;
end;

// ----------------------------------------------------------------------------                .
function sysGetFileSize (const file_name: String): Integer;
var
   sr: TSearchRec;
begin
   Assert( (Trim(file_name) <> ''),
           'param <file_name> darf nicht leer sein' );
   Result := (-1);
   if (__int_GetFileInfo(file_name,sr)) then begin
      Result := sr.Size;
   end;
end;

// ----------------------------------------------------------------------------                .
function sysGetSystemDirectory (): String;
var
   buf: array [0..MAX_PATH] of Char;
begin
   if (GetSystemDirectory(buf,SizeOf(buf)) <> 0) then begin
      Result := Trim(StrPas(buf));
      if (Result[Length(Result)] <> '\') then begin
         Result := (Result + '\');
      end;
   end else begin
      Result := ''; // GetSystemDirectory() ist fehlgeschlagen                                 .
   end;
end;

// 25.08.'04: Fix: "wenn (len = 0) dann war's erfolgreich" war .. falsch :(                    .
// ----------------------------------------------------------------------------                .
function sysGetTempDirectory (): String;
var
   buf: array [0..MAX_PATH] of Char;
   len: DWORD;
begin
   len := GetTempPath(SizeOf(buf),buf);
   if ((len > 0) and (len <= SizeOf(buf))) then begin
      Result := StrPas(buf);
      if (Trim(Result) = '') then begin
         Result := 'c:'; // 'c:' gibt's ein einem Windows-System immer ..                      .
      end;
   end else begin
      Result := 'c:';    // -"-                                        ..                      .
   end;
   if (Result[Length(Result)] <> '\') then begin
      Result := (Result + '\');
   end;
end;

// ----------------------------------------------------------------------------                .
function sysGetTempFilename (): String;
var
   buf: array [0..MAX_PATH] of Char;
begin
   if (Windows.GetTempFileName(
          PChar(sysGetTempDirectory()),    // address of directory name for temporary file     .
                                           // address of filename prefix                       .
          PChar(ExtractFileName(ParamStr(0) + '.')),
          0,                               // number used to create temporary filename         .
          buf) <> 0) then begin            // address of buffer that receives the new filename .
      Result := Trim(StrPas(buf));
   end else begin
      Result := ('c:' + '\tempfile.tmp');
   end;
end;

// ----------------------------------------------------------------------------                .
function sysGetWin32ComputerName (): String;
var
   buf: array [Byte] of Char;
   buf_size: Cardinal;
begin
   buf_size := SizeOf(buf);
   if (GetComputerName(buf,buf_size)) then begin
      Result := Trim(StrPas(buf));
   end else begin
      Result := '';
   end;
end;

// ----------------------------------------------------------------------------                .
function sysGetWin32UserName (): String;
var
   buf: array [Byte] of Char;
   buf_size: Cardinal;
begin
   buf_size := SizeOf(buf);
   if (GetUserName(buf,buf_size)) then begin
      Result := Trim(StrPas(buf));
   end else begin
      Result := '';
   end;
end;

// ----------------------------------------------------------------------------                .
function sysGetWin32Version (): TWin32VersionResult;
var
   w32_VerName, w32_CSDVerName: String;
begin
   case Win32Platform of
      VER_PLATFORM_WIN32s: begin            // --- Win 32s on Win3.x    ---------------------  .
         w32_VerName := 'Win32s/Win3.x';
      end;
      VER_PLATFORM_WIN32_WINDOWS: begin     // --- Windows 95 ~ 98 ~ ME ---------------------  .
         case Win32MinorVersion of
            0: begin                        // Windows 95                                      .
               w32_VerName := 'Win32/Win95';
            end;
            10: begin                       // Windows 98 / 98SE                               .
               if (Win32CSDVersion = '') then
                  w32_VerName := 'Win32/Win98'
               else
                  w32_VerName := 'Win32/Win98SE';
            end;
            90: begin                       // Windows ME                                      .
               w32_VerName := 'Win32/ME';
            end;
            else begin                      // nicht bestimmbar                                .
               w32_VerName := Format('ID-%d',[Win32Platform]);
            end;
         end; { case Win32MinorVersion }
      end;
      VER_PLATFORM_WIN32_NT: begin          // --- Windows NT ~ 2k ~ XP ---------------------  .
         case Win32MajorVersion of
            4: begin                        // Windows NT 4                                    .
               w32_VerName := 'Win32/WinNT4';
            end;
            5: begin                        // WIndows 2k / XP                                 .
               if (Win32MinorVersion = 0) then begin
                  w32_VerName := 'Win32/Win2000';
               end else if (Win32MinorVersion = 1) then begin
                  w32_VerName := 'Win32/WinXP';
               end else begin               // nicht bestimmbar                                .
                  w32_VerName := Format('ID-%d',[Win32Platform]);
               end;
            end;
            else begin                      // nicht bestimmbar                                .
               w32_VerName := Format('ID-%d',[Win32Platform]);
            end;
         end; { case Win32MajorVersion }
      end;
      else begin                            // --- no idea at all       ---------------------  .
         w32_VerName := Format('ID-%d',[Win32Platform]);
      end;
   end; { case Win23Platform }
   if (Win32CSDVersion = '') then begin
      w32_CSDVerName := '(n/a)';
   end else begin
      w32_CSDVerName := Win32CSDVersion;
   end;
   if (Win32Platform = VER_PLATFORM_WIN32_NT) then begin
      Result.w32Name         := w32_VerName;
      Result.w32CSDName      := w32_CSDVerName;
      Result.w32MajorVersion := Byte(Win32MajorVersion); // Integer -> Byte                    .
      Result.w32MinorVersion := Byte(Win32MinorVersion); // Integer -> Byte                    .
      Result.w32BuildNumber  := Word(Win32BuildNumber);  // Integer -> Word                    .
      Result.w32ComposedName := Format('%s (%d.%d.%d) - %s',
                                       [w32_VerName,
                                        Result.w32MajorVersion,Result.w32MinorVersion,
                                        Result.w32BuildNumber,w32_CSDVerName]);
   end else if (Win32Platform = VER_PLATFORM_WIN32_WINDOWS) then begin
      Result.w32Name         := w32_VerName;
      Result.w32CSDName      := w32_CSDVerName;
      // (from the win32 help file):                                                           .
      // dwBuildNumber                                                                         .
      //   Windows NT (dwPlatformId = VER_PLATFORM_WIN32_NT):                                  .
      //     Identifies the build number of the operating system.                              .
      //   Windows 95 (dwPlatformId = VER_PLATFORM_WIN32_WINDOWS):                             .
      //     Identifies the build number of the operating system in the low-order              .
      //     word. The high-order word contains the major and minor version numbers.           .
      // --                                                                                    .
      // While this might be true for a "real" win95/98 system, if you "fake" the os by using  .
      //   'Windows 2000 Support Tools' > 'Tools' > 'Application Compatibility Tool'           .
      // then HIWORD(Win32BuildNumber) is zero. Deal with that ..                              .
      if (LongWordRec(Win32BuildNumber).HiHi > 0) then begin
         Result.w32MajorVersion := LongWordRec(Win32BuildNumber).HiHi; // -> liefert Byte      .
         Result.w32MinorVersion := LongWordRec(Win32BuildNumber).HiLo; // -> liefert Byte      .
      end else begin
         Result.w32MajorVersion := Byte(Win32MajorVersion);            // Integer -> Byte      .
         Result.w32MinorVersion := Byte(Win32MinorVersion);            // Integer -> Byte      .
      end;
      Result.w32BuildNumber  := LongRec(Win32BuildNumber).Lo;          // -> liefert Word      .
      Result.w32ComposedName := Format('%s (%d.%.2d.%d) - %s',
                                       [w32_VerName,
                                        Result.w32MajorVersion,Result.w32MinorVersion,
                                        Result.w32BuildNumber,w32_CSDVerName]);
   end else begin // (Win32Platform = VER_PLATFORM_WIN32s)                                     .
      Result.w32Name         := w32_VerName;
      Result.w32CSDName      := w32_CSDVerName;
      Result.w32MajorVersion := 0;
      Result.w32MinorVersion := 0;
      Result.w32BuildNumber  := 0;
      Result.w32ComposedName := Format('%s (%d.%d.%d) - %s',
                                       [w32_VerName,
                                        Win32MajorVersion,Win32MinorVersion,
                                        Win32BuildNumber,w32_CSDVerName]);
   end;
end;

// ----------------------------------------------------------------------------                .
function sysGetWindowsDirectory (): String;
var
   buf: array [0..MAX_PATH] of Char;
begin
   if (GetWindowsDirectory(buf,SizeOf(buf)) <> 0) then begin
      Result := Trim(StrPas(buf));
      if (Result[Length(Result)] <> '\') then begin
         Result := (Result + '\');
      end;
   end else begin
      Result := ''; // GetWindowsDirectory() ist fehlgeschlagen                                .
   end;
end;

// ----------------------------------------------------------------------------                .
function sysShowErrMsg (const error_message: String): Integer;
begin
   Assert( (error_message <> ''),
           'param <error_message> darf nicht leer sein' );
   Result := Windows.MessageBox(0,PChar(error_message),PChar(ParamStr(0)),(MB_OK or MB_ICONERROR));
end;

// ----------------------------------------------------------------------------                .
// Interne Funktion; Wrapper fr sysSpawnProcess(), sysSpawnProcessAndWait()                   .
// ----------------------------------------------------------------------------                .
function __int_SpawnProcessAndWait (const image_file, run_parameters, work_directory: String; const wait_completition: Boolean): TSpawnResult;
var
   si: TStartupInfo;
   pi: TProcessInformation;
begin
   Assert( (Trim(image_file) <> ''),
           'param <image_file> darf nicht leer sein' );
   FillChar(si,SizeOf(si),0);
   si.cb := SizeOf(TStartupInfo);
   if (CreateProcess(
       NIL,                                  // pointer to name of executable module           .
       PChar(image_file + ' ' + run_parameters), // pointer to command line string             .
       NIL,                                  // pointer to process security attributes         .
       NIL,                                  // pointer to thread security attributes          .
       False,                                // handle inheritance flag                        .
       NORMAL_PRIORITY_CLASS,                // creation flags                                 .
       NIL,                                  // pointer to new environment block               .
       PChar(work_directory),                // pointer to current directory name              .
       si,                                   // pointer to STARTUPINFO                         .
       pi)) then begin                       // pointer to PROCESS_INFORMATION                 .
      if (wait_completition) then begin      // --- _schon_ auf Ende warten ..                 .
         WaitForSingleObject(pi.hProcess,INFINITE);         // Warten, bis Prozess terminiert  .
         GetExitCodeProcess(pi.hProcess,Result.srExitCode); // Exit-Code: abrufen              .
      end else begin                         // --- _nicht_ auf Ende warten ..                 .
         Result.srExitCode := Cardinal(-1);                 // Exit-Code: nicht verfgbar      .
      end;
      CloseHandle(pi.hProcess);              // Prozess-Handle schlieen                       .
      Result.srResult := 0;                  // CreateProcess() war erfolgreich                .
                                             // 'kurz gesagt'-Status: ermitteln ..             .
      Result.srShortResult := ((not(wait_completition)) or
                               (Result.srExitCode = 0));
   end else begin
      Result.srExitCode := Cardinal(-1);     // Exit-Code: nicht verfgbar                     .
      Result.srResult := (-1);               // CreateProcess() ist fehlgeschlagen             .
      Result.srShortResult := False;         // 'kurz gesagt'-Status: nein, sorry              .
   end;
end;

// ----------------------------------------------------------------------------                .
function sysSpawnProcess (const image_file, run_parameters, work_directory: String): TSpawnResult;
begin
   Assert( (Trim(image_file) <> ''),
           'param <image_file> darf nicht leer sein' );
   Result := __int_SpawnProcessAndWait(image_file,run_parameters,work_directory,False);
end;

// ----------------------------------------------------------------------------                .
function sysSpawnProcessAndWait (const image_file, run_parameters, work_directory: String): TSpawnResult;
begin
   Assert( (Trim(image_file) <> ''),
           'param <image_file> darf nicht leer sein' );
   Result := __int_SpawnProcessAndWait(image_file,run_parameters,work_directory,True);
end;

// ----------------------------------------------------------------------------                .
function sysStartMailClient (const mail_to, mail_subject: String; const mail_body: String; const mail_attachments: array of String): Boolean;
var
   cmd_line: String;
   i: Integer;
begin
   Assert( (Trim(mail_to) <> ''),
           'param <mail_to> darf nicht leer sein' );
{$IFDEF DelphiPascalCompiler}
   cmd_line := ('mailto:' + strJsEscape(mail_to) + '?subject=' + strJsEscape(mail_subject));
{$ELSE} {$IFDEF FreePascalCompiler}
   cmd_line := ('mailto:' + strJsEscape(mail_to,False) + '?subject=' + strJsEscape(mail_subject,False));
{$ENDIF} {$ENDIF}
   if (mail_body <> '') then begin
   {$IFDEF DelphiPascalCompiler}
      cmd_line := (cmd_line + '&body=' + strJsEscape(mail_body));
   {$ELSE} {$IFDEF FreePascalCompiler}
      cmd_line := (cmd_line + '&body=' + strJsEscape(mail_body,False));
   {$ENDIF} {$ENDIF}
   end;
   for i := 0 to High(mail_attachments) do begin
      if (Trim(mail_attachments[i]) <> '') then begin
      {$IFDEF DelphiPascalCompiler}
         cmd_line := (cmd_line + '&file=' + strJsEscape(mail_attachments[i]));
      {$ELSE} {$IFDEF FreePascalCompiler}
         cmd_line := (cmd_line + '&file=' + strJsEscape(mail_attachments[i],False));
      {$ENDIF} {$ENDIF}
      end;
   end;
   Result := (ShellExecute(0,'open',PCHAR(cmd_line),NIL,NIL,SW_SHOWNORMAL) > 32);
end;

// ----------------------------------------------------------------------------                .
// UTC_time = (local_time + System.bias)                                                       .
// ----------------------------------------------------------------------------                .
{$IFDEF DelphiPascalCompiler}
function timeConvertLocalTimeToUTC (const datetime: TDateTime; const NULL_onerror: Boolean = False): TDateTime;
{$ELSE} {$IFDEF FreePascalCompiler}
function timeConvertLocalTimeToUTC (const datetime: TDateTime; const NULL_onerror: Boolean): TDateTime;
{$ENDIF} {$ENDIF}
var
   tzi: TTimeZoneInformation;
   erg: Cardinal;
begin
   erg := GetTimeZoneInformation(tzi);
   case erg of
      TIME_ZONE_ID_STANDARD: begin // AKA "Winterzeit"                                         .
         Result := (datetime + ((tzi.Bias + tzi.StandardBias) / (24.0 * 60.0)));
      end;
      TIME_ZONE_ID_DAYLIGHT: begin // AKA "Sommerzeit"                                         .
         Result := (datetime + ((tzi.Bias + tzi.DaylightBias) / (24.0 * 60.0)));
      end;
      else begin             // Fehler oder keine TimeZone-Information ..                      .
         if (NULL_onerror) then begin
            Result := 0.0;
         end else begin
            Result := datetime;
         end;
      end;
   end;
end;

// ----------------------------------------------------------------------------                .
// local_time = (UTC_time - System.bias )                                                      .
// ----------------------------------------------------------------------------                .
{$IFDEF DelphiPascalCompiler}
function timeConvertUTCToLocalTime (const datetime: TDateTime; const NULL_onerror: Boolean = False): TDateTime;
{$ELSE} {$IFDEF FreePascalCompiler}
function timeConvertUTCToLocalTime (const datetime: TDateTime; const NULL_onerror: Boolean): TDateTime;
{$ENDIF} {$ENDIF}
var
   tzi: TTimeZoneInformation;
   erg: Cardinal;
begin
   erg := GetTimeZoneInformation(tzi);
   case erg of
      TIME_ZONE_ID_STANDARD: begin // AKA "Winterzeit"                                         .
         Result := (datetime - ((tzi.Bias + tzi.StandardBias) / (24.0 * 60.0)));
      end;
      TIME_ZONE_ID_DAYLIGHT: begin // AKA "Sommerzeit"                                         .
         Result := (datetime - ((tzi.Bias + tzi.DaylightBias) / (24.0 * 60.0)));
      end;
      else begin             // Fehler oder keine TimeZone-Information ..                      .
         if (NULL_onerror) then begin
            Result := 0.0;
         end else begin
            Result := datetime;
         end;
      end;
   end;
end;

// ----------------------------------------------------------------------------                .
// From the documentation at <http://www.swatch.com/>:                                         .
//                                                                                             .
// Where is the internet time meridian?                                                        .
//                                                                                             .
// Biel Meantime (BMT) ist the universal reference for internet time. A day in internet time   .
// begins at midnight BMT (@000 swatch .BEATs) (= CET = Central European [Winter]Time). The    .
// meridian is marked for all to see on the facade of the swatch international heaquarters     .
// on Jakob-Staempfli Street, Biel, Switzerland.                                               .
// ----------------------------------------------------------------------------                .
// Update 01.08.2005 @ 10:18                                                                   .
// Info ist nicht mehr direkt zu finden; lt. google.com war (ist?) die URL                     .
//    http://www.swatch.com/countryworld/internettime_english.php                              .
// ; der Inhalt ist derzeit nur ber den google.com - Cache zugnglich.                        .
// ----------------------------------------------------------------------------                .
function timeConvertLocalTimeToSwatchBEATs (const datetime: TDateTime): Word;
var
   tzi: TTimeZoneInformation;
   erg: Cardinal;
begin
   erg := GetTimeZoneInformation(tzi);
   case erg of
      TIME_ZONE_ID_STANDARD,       // AKA "Winterzeit" | Fr beide Werte gilt: Bezug           .
      TIME_ZONE_ID_DAYLIGHT: begin // AKA "Sommerzeit" | fr Swatch .BEATs ist Winterzeit ..   .
         Result := Word(Trunc((1000.0 / 24.0) + (Frac(datetime + ((tzi.Bias + tzi.StandardBias) / (24.0 * 60.0))) * 1000.0)));
      end;
      else begin
         Result := Word(Trunc(                   Frac(datetime                                                  ) * 1000.0) );
      end;
   end;
end;

// ----------------------------------------------------------------------------                .
function timeConvertSwatchBEATsToLocalTime (const beats: Smallint): TDateTime;
begin
   Assert( ((beats >= 0) and (beats < 1000)),
           'param <beats> ist nicht im Bereich von [0..999]' );
   Result := 0.0;
end;

// ----------------------------------------------------------------------------                .
function Sieve_Of_Eratosthenes (const N: Integer; var A: TSieveResultArray): Integer;
var
   i, j, p: Integer;
begin
   Assert( (N >= 1),
           'param <N> darf nicht negativ oder Null sein' );
   SetLength(A,(N + 1));          // ein dyn. Array beginnt bei [0], daher (N+1) ..            .
   A[1] := 0;                     // 1 ist keine Primzahl ..                                   .
   Result := (N - 1);             // so viele Primzahlen erwarten wir maximal ..               .
   for i := 2 to N do begin
      A[i] := 1;                  // potentielle Primzahlen markieren ..                       .
   end;
   p := 2;
   while ((p * p) <=  N) do begin // das ist: "p <= Quadratwurzel(N)", nur ohne Fliekomma ..  .
      j := (2 * p);
      while (j  <=  N) do begin
         if (A[j] = 1) then begin
            A[j] := 0;            // das ist doch keine Primzahl ..                            .
            Dec(Result);          // somit auch (Gesamtzahl - 1) ..                            .
         end;
         j := (j + p);
      end;
      p := (p + 1);               // damit machen wir weiter ..                                .
      while (A[p] = 0) do begin
         p := (p + 1);            // .. und berspringen zustzlich alle Nicht-Primzahlen      .
      end;
   end;
end;

(*
 *                                                                                             .
 * ============================================================================                .
 * ===  Initialisierung der Unit                                            ===                .
 * ============================================================================                .
 *                                                                                             .
 *)

// ---------------                                                                             .
   initialization
// ---------------                                                                             .

begin
   // Initialisierung -- Start.                                                                .

   Randomize();

   // Work-Around fr z.B. Win98 (Standard), wo (Win32CSDVersion = ' ') ist!                   .
   Win32CSDVersion := Trim(Win32CSDVersion);

{$IFDEF CONSOLE}
   AssignFile(Error,'');
   ReWrite(Error);
   TTextRec(Error).Handle := GetStdHandle(STD_ERROR_HANDLE);
{$ENDIF}

   // Initialisierung -- Ende.                                                                 .
end;

// -------------                                                                               .
   finalization
// -------------                                                                               .

begin
   // Finalisierung -- Start.                                                                  .

{$IFDEF CONSOLE}
   CloseFile(Error);
{$ENDIF}

   // Finalisierung -- Ende.                                                                   .
end;

end.

