(*
 *                                                                                             .
 * Module name: nfGUI_u.pas; Module type: Delphi unit; Language: Object pascal;                .
 * Original developed with: Delphi 5 Enterprise; Portability: most likely down to Delphi 2;    .
 *                                                                                             .
 * Author: ~SoffyT (web:<http://www.nefzger.at/~SoffyT/>; mailto:<MissyQueen@nefzger.at>)      .
 * File created: Tue., 2003-07-01; Last modified: Sun., 2005-10-23                             .
 *                                                                                             .
 * Bug report: Report all bugs, wishes, suggestions etc. to <BugReport@nefzger.at>.            .
 *                                                                                             .
 *  Purpose:                                                                                   .
 * ----------                                                                                  .
 *    GUI wrapper for nf.dpr (which is a console wrapper for Fakultaet.pas)                    .
 *                                                                                             .
 *)
unit
   nfGUI_u;

// ----------                                                                                  .
   interface
// ----------                                                                                  .

uses
   Windows,
   SysUtils,
   Messages,
   Classes,
   Graphics,
   Controls,
   Forms,
   Dialogs;

type
   TFnfGUIHelper = class (TForm)
      procedure FormCreate (Sender: TObject);
      procedure FormDestroy (Sender: TObject);
      procedure FormKeyDown (Sender: TObject; var Key: Word; Shift: TShiftState);
      procedure FormKeyUp (Sender: TObject; var Key: Word; Shift: TShiftState);
   private
   { Private-Deklarationen }
      procedure WMSYSCOMMAND (var Msg: TWMSYSCOMMAND); message WM_SYSCOMMAND;
   public
   { Public-Deklarationen }
   end;

var
   FnfGUIHelper : TFnfGUIHelper;

// ---------------                                                                             .
   implementation
// ---------------                                                                             .

{$R *.DFM}

uses
   Misc_Utilities,
   Defs,
   nfGUI_u_Frame,
   IniFiles;

const
   CMD_ABOUT       = $4711;
   CMD_USAGE       = $4712;
   CMD_MAILCOMMENT = $4713;
   CMD_MAILBUG     = $4714;

// ----------------------------------------------------------------------------                .
procedure TFnfGUIHelper.WMSYSCOMMAND (var Msg: TWMSYSCOMMAND);
begin
   Msg.Result := 0;
   case Msg.CmdType of
      CMD_ABOUT: begin
         FnfGUIHelper_Frame.btn_About.Click();
      end;
      CMD_USAGE: begin
         FnfGUIHelper_Frame.btn_About2.Click();
      end;
      CMD_MAILCOMMENT: begin
         sysStartMailClient(
            'MissyQueen@nefzger.at',
            ('[nf GUI]' + ' -- ' + '[' + Caption + ']' + ' -- I have a comment ..'),
            (CRCR + '=== Pls. don''t delete anything below this line! ===' + CRCR +
             TAB + sysGetWin32Version().w32ComposedName + CRCR +
             FnfGUIHelper_Frame.get_nfGUI_exe_Info + CR + FnfGUIHelper_Frame.get_nf_exe_Info),
            []);
      end;
      CMD_MAILBUG: begin
         sysStartMailClient(
            'BugReport@nefzger.at',
            ('[nf GUI]' + ' -- ' + '[' + Caption + ']' + ' -- I have found a bug ..'),
            (CRCR + '=== Pls. don''t delete anything below this line! ===' + CRCR +
             TAB + sysGetWin32Version().w32ComposedName + CRCR +
             FnfGUIHelper_Frame.get_nfGUI_exe_Info + CR + FnfGUIHelper_Frame.get_nf_exe_Info),
            []);
      end;
      else begin
         inherited; // DefWindowProc(Handle,Msg.Msg,Msg.wParam,Msg.lParam);
      end;
   end;
end;

// ----------------------------------------------------------------------------                .
procedure TFnfGUIHelper.FormCreate (Sender: TObject);
var
   ini : TIniFile;
   h : HMENU;
begin
   // Fenster-Setup #1                                                                         .
   ini := TIniFile.Create(ChangeFileExt(ParamStr(0),'.ini'));
   try
      Left := ini.ReadInteger((NF_GUI_NAME + '.' + sysGetWin32UserName()),'Left',((Screen.Width -Width ) div 2));
      Top  := ini.ReadInteger((NF_GUI_NAME + '.' + sysGetWin32UserName()),'Top' ,((Screen.Height-Height) div 2));
   finally
      FreeAndNil(ini);
   end;
   // Frame-Setup                                                                              .
   FnfGUIHelper_Frame := TFnfGUIHelper_Frame.Create(Self);
   FnfGUIHelper_Frame.Parent := Self;
   FnfGUIHelper_Frame.Frame_Create(Self);
   with FnfGUIHelper_Frame do begin
      Left := 0;
      Top  := 0;
   end;
   // Fenster-Setup #2                                                                         .
   BorderStyle := bsSizeable;
   Position := poDesigned;
   ActiveControl := FnfGUIHelper_Frame.btn_Calculate;
   Caption := Format('nf GUI helper %s',[resGetXVersionInfo()]);
   // System-Men anpassen                                                                     .
   h := GetSystemMenu(Handle,False);
   if (h <> 0) then begin
      AppendMenu(h,MF_SEPARATOR,0,NIL);
      AppendMenu(h,MF_STRING,CMD_ABOUT,'&About && version info'#9'F1');
      AppendMenu(h,MF_STRING,CMD_USAGE,'Usage for ''&' + NF_EXE_NAME + ''''#9'Alt+F1');
      AppendMenu(h,MF_SEPARATOR,0,NIL);
      AppendMenu(h,MF_STRING,CMD_MAILCOMMENT,'Mail the a&uthor: MissyQueen@nefzger.at');
      AppendMenu(h,MF_STRING,CMD_MAILBUG,'Sub&mit a bug: BugReport@nefzger.at');
      DrawMenuBar(Handle);
   end;
end;

// ----------------------------------------------------------------------------                .
procedure TFnfGUIHelper.FormDestroy (Sender: TObject);
var
   ini : TIniFile;
begin
   ini := TIniFile.Create(ChangeFileExt(ParamStr(0),'.ini'));
   // fr 'nfGUI' selbst ...                                                                   .
   try
      ini.WriteInteger((NF_GUI_NAME + '.' + sysGetWin32UserName()),'Left',Left);
      ini.WriteInteger((NF_GUI_NAME + '.' + sysGetWin32UserName()),'Top' ,Top );
   finally
      FreeAndNil(ini);
   end;
   FnfGUIHelper_Frame.Frame_Destroy(Self);
end;

// ----------------------------------------------------------------------------                .
procedure TFnfGUIHelper.FormKeyDown (Sender: TObject; var Key: Word; Shift: TShiftState);
begin
   if ((Shift = []) and (Key = VK_ESCAPE)) then begin
      Close();
   end;
end;

// ----------------------------------------------------------------------------                .
procedure TFnfGUIHelper.FormKeyUp (Sender: TObject; var Key: Word; Shift: TShiftState);
begin
   FnfGUIHelper_Frame.Frame_KeyUp(Self,Key,Shift);
end;

   // keine Initialisierung.                                                                   .
end.

