(*
 *                                                                                             .
 * Module name: nfGUI_u_Frame.pas; Module type: Delphi (frame) unit; Language: Object pascal;  .
 * Original developed with: Delphi 5 Enterprise; Portability: most likely down to Delphi 2;    .
 *                                                                                             .
 * Author: ~SoffyT (web:<http://www.nefzger.at/~SoffyT/>; mailto:<MissyQueen@nefzger.at>)      .
 * File created: Tue., 2003-07-01; Last modified: Tue., 2005-11-01                             .
 *                                                                                             .
 * Bug report: Report all bugs, wishes, suggestions etc. to <BugReport@nefzger.at>.            .
 *                                                                                             .
 *  Purpose:                                                                                   .
 * ----------                                                                                  .
 *    GUI wrapper for nf.dpr (which is a console wrapper for Fakultaet.pas)                    .
 *                                                                                             .
 *)
unit
   nfGUI_u_Frame;

// ----------                                                                                  .
   interface
// ----------                                                                                  .

uses
   Windows,
   SysUtils,
   Messages,
   Classes,
   Graphics,
   Controls,
   StdCtrls,
   ExtCtrls,
   Buttons,
   Forms,
   Dialogs,
   Misc_Utilities;

const
   NF_GUI_NAME       = 'nfGUI.exe';
   NF_EXE_NAME       = 'nf.exe';
   NF_OUTPUTLOG_NAME = 'nf.output.log';

type
   TFnfGUIHelper_Frame = class (TFrame)
      Notebook1: TNotebook;
      nfGUI_Finish_Desc: TLabel;
      nfGUI_Finish_Bevel: TBevel;
      nfGUI_Finish_Number_Desc: TLabel;
      nfGUI_Finish_Number_Edit: TEdit;
      nfGUI_Finish_nfVersion: TLabel;
      Bevel1: TBevel;
      btn_About: TButton;
      btn_About2: TButton;
      btn_Calculate: TBitBtn;
      procedure btn_AboutClick (Sender: TObject);
      procedure btn_About2Click (Sender: TObject);
      procedure btn_CalculateClick (Sender: TObject);
   private
   { Private-Deklarationen }
      procedure Delete_OutputLog ({keine Parameter});
      function Call_Program (const UsageOnly: Boolean): TSpawnResult;
      function Read_Registry ({keine Parameter}): Boolean;
      function Write_Registry ({keine Parameter}): Boolean;
   public
   { Public-Deklarationen }
      procedure Frame_Create (Sender: TObject);
      procedure Frame_Destroy (Sender: TObject);
      procedure Frame_KeyUp (Sender: TObject; var Key: Word; Shift: TShiftState);
      function get_nfGUI_exe_Info ({keine Parameter}): String;
      function get_nf_exe_Info ({keine Parameter}): String;
   end;

var
   FnfGUIHelper_Frame : TFnfGUIHelper_Frame;

// ---------------                                                                             .
   implementation
// ---------------                                                                             .

{$R *.DFM}

uses
   Dialog_Utilities,
   Defs,
   IniFiles;

const
   DLG_OFS : TPoint
      = (X: 24; Y: 32);

// ----------------------------------------------------------------------------                .
procedure TFnfGUIHelper_Frame.Delete_OutputLog ();
begin
   if (FileExists(sysGetTempDirectory() + NF_OUTPUTLOG_NAME)) then begin
      if not(helperSafeDeleteFile(sysGetTempDirectory() + NF_OUTPUTLOG_NAME)) then begin
         // Fehler -- kann Output-Datei nicht lschen :-(                                      .
         MessageDlgPosEx(
            (Self.Owner as TForm),
            'Fatal:' + CRCR +
            'Could not delete file ''' + sysGetTempDirectory() + NF_OUTPUTLOG_NAME + '''.',
            mtError,[mbOK],0,DLG_OFS);
      end;
   end;
end;

// ----------------------------------------------------------------------------                .
function TFnfGUIHelper_Frame.Call_Program (const UsageOnly: Boolean): TSpawnResult;
var
   cmd_line: String;
begin
   if (UsageOnly) then begin
      // nur die "Usage"-Meldung anzeigen lassen ..                                            .
      cmd_line := Format(
         '/c %s%s > %s%s',
         [appGetApplicationPath(),NF_EXE_NAME,
          sysGetTempDirectory(),NF_OUTPUTLOG_NAME]);
   end else begin
      // normale Parameter, normale Operation ..                                               .
      cmd_line := Format(
         '/c %s%s %d %s > %s%s',
         [appGetApplicationPath(),NF_EXE_NAME,
          StrToIntDef(nfGUI_Finish_Number_Edit.Text,(-1)),'--quiet',
          sysGetTempDirectory(),NF_OUTPUTLOG_NAME]);
   end;
   // Programm lt. Parameter ausfhren                                                         .
   Result := sysSpawnProcessAndWait(
      'cmd.exe',
      cmd_line,
      appGetApplicationPath());
   // Ergebnis auswerten ...                                                                   .
   if (Result.srResult <> 0) then begin
      // Fehler - Funktion fehlgeschlagen                                                      .
      MessageDlgPosEx(
         (Self.Owner as TForm),
         'Fatal:' + CRCR +
         'Unable to create process from image file ''' + NF_EXE_NAME +
         ''' through ''cmd.exe''.',
         mtError,[mbOK],0,DLG_OFS);
   end else if (Result.srExitCode <> 0) then begin
      // Fehler - Funktion OK, aber ed.exe fehlgeschlagen                                      .
      MessageDlgPosEx(
         (Self.Owner as TForm),
         'Fatal:' + CRCR +
         'Process from image file ''' + NF_EXE_NAME +
         ''' (through ''cmd.exe'') failed to execute correctly.',
         mtError,[mbOK],0,DLG_OFS);
   end else begin
      // Erfolg                                                                                .
      ;
   end;
   if (FileExists(sysGetTempDirectory() + NF_OUTPUTLOG_NAME)) then begin
      if not(sysSpawnProcess(
                'notepad.exe',
                (sysGetTempDirectory() + NF_OUTPUTLOG_NAME),
                sysGetTempDirectory()).srShortResult) then begin
         MessageDlgPosEx(
            (Self.Owner as TForm),
            'Fatal:' + CRCR +
            'Couldn''t spawn the editor to display the result file ''' +
            sysGetTempDirectory() + NF_OUTPUTLOG_NAME + '''.',
            mtError,[mbOK],0,DLG_OFS);
      end;
   end else begin
      // Fehler: es gibt _immer_ eine Ausgabe von nf.exe                                       .
      MessageDlgPosEx(
         (Self.Owner as TForm),
         'Fatal:' + CRCR +
         'Something went *badly* wrong while executing ''' + NF_EXE_NAME + '''.',
         mtError,[mbOK],0,DLG_OFS);
   end;
end;

// ----------------------------------------------------------------------------                .
function TFnfGUIHelper_Frame.Read_Registry (): Boolean;
var
   ini : TIniFile;
begin
   ini := TIniFile.Create(ChangeFileExt(ParamStr(0),'.ini'));
   try
      try
         // Parameter fr 'nf'                                                                    .
         nfGUI_Finish_Number_Edit.Text := ini.ReadString((NF_EXE_NAME + '.' + sysGetWin32UserName()),'N','105');
         Result := True;
      except
         Result := False;
      end;
   finally
      FreeAndNil(ini);
   end;
end;

// ----------------------------------------------------------------------------                .
function TFnfGUIHelper_Frame.Write_Registry (): Boolean;
var
   ini : TIniFile;
begin
   ini := TIniFile.Create(ChangeFileExt(ParamStr(0),'.ini'));
   try
      try
         // Parameter fr 'nf'                                                                    .
         ini.WriteString((NF_EXE_NAME + '.' + sysGetWin32UserName()),'N',nfGUI_Finish_Number_Edit.Text);
         Result := True;
      except
         Result := False;
      end;
   finally
      FreeAndNil(ini);
   end;
end;

// ----------------------------------------------------------------------------                .
procedure TFnfGUIHelper_Frame.Frame_Create (Sender: TObject);
{var                                                     }
{   i : Integer;                                         }
begin
{   for i := 0 to (ComponentCount - 1) do begin          }
{      if (Components[i] is TWinControl) then begin      }
{         (Components[i] as TWinControl).HandleNeeded(); }
{      end;                                              }
{    end;                                                }
   NotebookHandlesNeeded(Notebook1);
   if (FileExists(appGetApplicationPath() + NF_EXE_NAME)) then begin
      nfGUI_Finish_nfVersion.Caption := Format('%s: %s',
         [NF_EXE_NAME,resGetXVersionInfo(appGetApplicationPath() + NF_EXE_NAME)]);
   end else begin
      nfGUI_Finish_nfVersion.Caption := Format('%s: %s',
         [NF_EXE_NAME,'**not installed**']);
   end;
   Delete_OutputLog();
   Read_Registry();
end;

// ----------------------------------------------------------------------------                .
procedure TFnfGUIHelper_Frame.Frame_Destroy (Sender: TObject);
begin
   Write_Registry();
   Delete_OutputLog();
end;

// ----------------------------------------------------------------------------                .
// allgemeiner Gotcha:                                                                         .
// Ein Dialog - via MessageDlg() geffnet - kann mittels ESC geschlossen werden; dies ist      .
// zwar schn, aber es ist zu beachten, da dies im Dialog als Reaktion auf KEYDOWN ge-        .
// schieht! Implikation: wenn eine Anwendung mittels ESC geschlossen werden kann, und die-     .
// se Reaktion in KEYUP (!) implementiert ist, so schlgt der Druck auf ESC nach dem Dia-      .
// log (-> keyDOWN) in die Applikation durch (-> keyUP) und beendet damit die Applikation.     .
// Work-Around: zumindest die 'auf ESC schlieen'-Behandlung mu in KEYDOWN erfolgen ...       .
// ----------------------------------------------------------------------------                .
procedure TFnfGUIHelper_Frame.Frame_KeyUp (Sender: TObject; var Key: Word; Shift: TShiftState);
begin
   if ((Shift = []) and (Key = VK_F1)) then begin
      Key := 0;
      btn_About.Click();
   end else if ((Shift = [ssAlt]) and (Key = VK_F1)) then begin
      Key := 0;
      btn_About2.Click();
   end;
end;

// -------------------------------------------------------------------------                   .
function TFnfGUIHelper_Frame.get_nfGUI_exe_Info ({keine Parameter}): String;
begin
   Result := TAB + '[ ----- ''' + NF_GUI_NAME + ''' ----- ]' + CR +
             TAB + '  version' + TAB + resGetXVersionInfo() + CR +
             TAB + '  release' + TAB + resGetXReleaseInfo() + CR +
             TAB + '  datetime' + TAB + resGetXDateTimeInfo();
end;

// -------------------------------------------------------------------------                   .
function TFnfGUIHelper_Frame.get_nf_exe_Info ({keine Parameter}): String;
begin
   Result := TAB + '[ ----- ''' + NF_EXE_NAME + ''' ----- ]' + CR;
   if (FileExists(appGetApplicationPath() + NF_EXE_NAME)) then begin
      Result := Result +
                TAB +'  version' + TAB + resGetXVersionInfo(appGetApplicationPath() + NF_EXE_NAME) + CR +
                TAB +'  release' + TAB + resGetXReleaseInfo(appGetApplicationPath() + NF_EXE_NAME) + CR +
                TAB +'  datetime' + TAB + resGetXDateTimeInfo(appGetApplicationPath() + NF_EXE_NAME);
   end else begin
      Result := Result +
                TAB + '  **not installed**';
   end;
end;

// ----------------------------------------------------------------------------                .
procedure TFnfGUIHelper_Frame.btn_AboutClick (Sender: TObject);
begin
   Screen.Cursor := crHourGlass;
   try
      MessageDlgPosEx(
         (Self.Owner as TForm),
         'About ''' + NF_GUI_NAME + ''', ''' + NF_EXE_NAME + ''' && version info:' + CRCR +
         get_nfGUI_exe_Info() + CRCR +
         get_nf_exe_Info() + CRCR +
         resGetXCopyrightInfo() + CR +
         'Running on: ' + sysGetWin32Version().w32ComposedName,
         mtInformation,[mbOK],0,DLG_OFS);
   finally
      Screen.Cursor := crDefault;
   end;
end;

// ----------------------------------------------------------------------------                .
procedure TFnfGUIHelper_Frame.btn_About2Click (Sender: TObject);
begin
   Screen.Cursor := crHourGlass;
   try
      // Programm ohne Parameter ausfhren -> Usage anzeigen ...                               .
      Delete_OutputLog();
      Call_Program(True);
   finally
      Screen.Cursor := crDefault;
   end
end;

// ----------------------------------------------------------------------------                .
procedure TFnfGUIHelper_Frame.btn_CalculateClick (Sender: TObject);
var
   erg : TSpawnResult;
begin
   Screen.Cursor := crHourGlass;
   btn_Calculate.Enabled := False;
   try
      // Programm lt. Parameter ausfhren                                                      .
      Delete_OutputLog();
      erg := Call_Program(False);
   finally
      btn_Calculate.Enabled := True;
      btn_Calculate.SetFocus();
      Screen.Cursor := crDefault;
   end;
end;

   // keine Initialisierung.                                                                   .
end.

